﻿using System;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using UrzaGatherer.ViewModels;
using UrzaIncubator;

namespace DatabaseRestore
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            if (args.Length != 1)
                return;

            string backupPath = args[0];

            if (!File.Exists(backupPath))
                return;

            DataRoot.WorkingPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "UrzaData");

            using (WaitForm frm = new WaitForm())
            {
                frm.Show();
                Application.DoEvents();
                RestoreBackup(backupPath);
                Process.Start("UrzaGatherer.exe");
            }
        }

        static void RestoreBackup(string backupPath)
        {
            try
            {
                RootViewModel.Restore(backupPath);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, @"Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }            
        }
    }
}
