﻿using System;
using System.Windows.Controls;
using System.Windows;

namespace Delta.Tools
{
    public class ExpandableTab : Button
    {
        public enum TabPositions
        {
            Top,
            Left,
            Bottom
        }

        public static DependencyProperty HeaderProperty = DependencyProperty.Register("Header", typeof(string), typeof(ExpandableTab));
        public static DependencyProperty TabPositionProperty = DependencyProperty.Register("TabPosition", typeof(TabPositions), typeof(ExpandableTab), new PropertyMetadata(TabPositionPropertyChanged));

        static void TabPositionPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ExpandableTab tab = (ExpandableTab)d;

            SetCorrectStyle(tab, (TabPositions)e.NewValue);
        }

        static void SetCorrectStyle(ExpandableTab tab, TabPositions position)
        {
            ResourceDictionary resourceDictionary = new ResourceDictionary();

            resourceDictionary.Source = new Uri(@"/Delta.Tools;component/Themes/Generic.xaml", UriKind.Relative);

            switch (position)
            {
                case TabPositions.Top:
                    tab.Style = (Style)resourceDictionary["TopExpandableTab"];
                    break;
                case TabPositions.Left:
                    tab.Style = (Style)resourceDictionary["LeftExpandableTab"];
                    break;
                case TabPositions.Bottom:
                    tab.Style = (Style)resourceDictionary["BottomExpandableTab"];
                    break;
            }
        }

        static ExpandableTab()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(ExpandableTab), new FrameworkPropertyMetadata(typeof(ExpandableTab)));
        }

        public ExpandableTab()
        {
            SetCorrectStyle(this, TabPositions.Top);
        }

        public object Header
        {
            get
            {
                return GetValue(HeaderProperty);
            }
            set
            {
                SetValue(HeightProperty, value);
            }
        }

        public TabPositions TabPosition
        {
            get
            {
                return (TabPositions)GetValue(TabPositionProperty);
            }
            set
            {
                SetValue(TabPositionProperty, value);
            }
        }
    }
}
