﻿using System.Linq;
using UrzaIncubator;

namespace UrzaGatherer.ViewModels
{
    public class ReferencesViewModel : ViewModelBase
    {
        IOrderedQueryable<Color> colors;
        IOrderedQueryable<Rarity> rarities;
        IOrderedQueryable<Author> authors;

        int currentAuthorID = -1;

        public IOrderedQueryable<Color> Colors
        {
            get
            {
                if (colors == null)
                {

                    colors = DataRoot.Entities.Colors.OrderBy(a => a.Order);
                }

                return colors;
            }
        }

        public IOrderedQueryable<Rarity> Rarities
        {
            get
            {
                if (rarities == null)
                {
                    rarities = DataRoot.Entities.Rarities.OrderBy(a => a.Order);
                }

                return rarities;
            }
        }

        public IOrderedQueryable<Author> Authors
        {
            get
            {
                if (authors == null)
                {
                    authors = DataRoot.Entities.Authors.OrderBy(a => a.Name);
                }

                return authors;
            }
        }


        public Author GetAuthorFromName(string authorName)
        {
            var queryAuthor = from authordb in DataRoot.Entities.Authors
                              where string.Compare(authordb.Name, authorName, true) == 0
                              select authordb;

            if (queryAuthor.Count() > 0)
            {
                return queryAuthor.First();
            }

            if (currentAuthorID == -1)
                currentAuthorID = (from a in DataRoot.Entities.Authors select a.ID).Max();

            Author newAuthor = new Author { ID = ++currentAuthorID, Name = authorName };

            DataRoot.Entities.AddToAuthors(newAuthor);
            return newAuthor;
        }

        public Color GetColorFromName(string colorName)
        {
            if (colorName == "Colorless")
                colorName = "Artifact";
            else if (string.IsNullOrEmpty(colorName))
                colorName = "Artifact";
            else if (colorName.Contains('/'))
                colorName = "Multicolor";

            var query = from color in DataRoot.Entities.Colors
                        where string.Compare(color.Name, colorName, true) == 0
                        select color;

            return query.First();
        }

        public int GetRarityFromName(string rarityName)
        {
            switch (rarityName.Substring(0, 1))
            {
                case "M":
                    return 4;
                case "R":
                    return 3;
                case "U":
                    return 2;
                default:
                    return 1;
            }
        }
    }
}
