﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Objects;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows.Media.Imaging;
using ICSharpCode.SharpZipLib.Zip;
using UrzaIncubator;
using UrzaIncubator.JSON;

namespace UrzaGatherer.ViewModels
{
    public class RootViewModel : ViewModelBase
    {
        BlockViewModel blockViewModel;
        ExpansionViewModel expansionViewModel;
        ChartViewModel chartViewModel;
        ReferencesViewModel referencesViewModel;

        readonly static DialogSystem dialogSystem = new DialogSystem();

        RelayCommand saveCommand;

        public BlockViewModel BlockViewModel
        {
            get { return blockViewModel; }
        }

        public ExpansionViewModel ExpansionViewModel
        {
            get { return expansionViewModel; }
        }

        public RelayCommand SaveCommand
        {
            get { return saveCommand; }
        }

        public static DialogSystem DialogSystem
        {
            get { return dialogSystem; }
        }

        public ChartViewModel ChartViewModel
        {
            get { return chartViewModel; }
        }

        public ReferencesViewModel ReferencesViewModel
        {
            get { return referencesViewModel; }
        }

        void Save()
        {
            int retry = 0;

            do
            {
                try
                {
                    Commit();
                    return;
                }
                catch (Exception)
                {

                    retry++;
                }
            }
            while (retry < 5);
        }

        public bool Prepare(bool isNetworkDeployed)
        {
            DataRoot.WorkingPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "UrzaData");

            if (!Directory.Exists(DataRoot.WorkingPath))
            {
                Directory.CreateDirectory(DataRoot.WorkingPath);
            }

            if (!File.Exists(Path.Combine(DataRoot.WorkingPath, "UrzaGatherer.sdf")))
            {
                if (isNetworkDeployed)
                {
                    return false;
                }
                File.Copy(Path.Combine(Environment.CurrentDirectory, "UrzaGatherer.sdf"), Path.Combine(DataRoot.WorkingPath, "UrzaGatherer.sdf"));
            }

            return true;
        }

        public void Start()
        {
            Tools.TestConnection();

            try
            {
                DataRoot.Entities = Entities.Create();
            }
            catch (Exception)
            {
                if (DataRoot.Entities != null)
                    DataRoot.Entities.Dispose();
                throw;
            }

            saveCommand = new RelayCommand(o => Save());

            blockViewModel = new BlockViewModel(this);
            expansionViewModel = new ExpansionViewModel(this);
            chartViewModel = new ChartViewModel(this);
            referencesViewModel = new ReferencesViewModel();
            return;
            var list = DataRoot.Entities.Expansions.First(ex => ex.Name.Contains("2014")).Cards.Where(c => c.ColorID == 7).ToList();

            //Array.Sort(cards, (card, card1) =>
            //    {
            //        if (card.ColorID == card1.ColorID)
            //        {
            //            return string.Compare(card.Name, card1.Name);
            //        }

            //        var colorOrder = card.ColorID;
            //        var colorOrder1 = card1.ColorID;

            //        switch (colorOrder)
            //        {
            //            case 1:
            //                colorOrder = 1;
            //                break;
            //            case 3:
            //                colorOrder = 2;
            //                break;
            //            case 5:
            //                colorOrder = 3;
            //                break;
            //            case 2:
            //                colorOrder = 5;
            //                break;
            //            case 4:
            //                colorOrder = 6;
            //                break;
            //            case 6:
            //                colorOrder = 7;
            //                break;
            //            case 7:
            //                colorOrder = 4;
            //                break;
            //            case 8:
            //                colorOrder = 8;
            //                break;
            //        }

            //        switch (colorOrder1)
            //        {
            //            case 1:
            //                colorOrder1 = 1;
            //                break;
            //            case 3:
            //                colorOrder1 = 2;
            //                break;
            //            case 5:
            //                colorOrder1 = 3;
            //                break;
            //            case 2:
            //                colorOrder1 = 5;
            //                break;
            //            case 4:
            //                colorOrder1 = 6;
            //                break;
            //            case 6:
            //                colorOrder1 = 7;
            //                break;
            //            case 7:
            //                colorOrder1 = 4;
            //                break;
            //            case 8:
            //                colorOrder1 = 8;
            //                break;
            //        }

            //        if (colorOrder1 == colorOrder)
            //            return 0;

            //        if (colorOrder > colorOrder1)
            //            return 1;

            //        return -1;
            //    });

            foreach (Card card in list)
            {
                Tools.GetCardImage2(card, "en");
                DataRoot.Entities.SaveChanges();

                Debug.WriteLine("{0} - {1}", card.Name, card.Expansion.Name);
            }

            return;


            //unlimited.Name = "Limited Edition Alpha";
            //unlimited = DataRoot.Entities.Expansions.Where(e => e.Name == "Beta").First();
            //unlimited.Name = "Limited Edition Beta";
            //DataRoot.Entities.SaveChanges();
            foreach (Card card in list)
            {
                if (card.CompletePath == null)
                {
                    if (!Tools.GetCardImage2(card, "en"))
                    {
                        Tools.GetCardImage2(card, "en", "a");
                    }

                    DataRoot.Entities.SaveChanges();
                    Debug.WriteLine("{0} / {1}", card.Name, card.Expansion.Name);
                }
            }
            return;
            //var carde = DataRoot.Entities.Cards.Where(c => c.Name == "Mox Lotus").First();
            //carde.Text = null;
            //carde.FrenchText = null;
            //Tools.GetCardInfos(carde);

            //foreach (Card card in list)
            //{
            //    if (card.FrenchText == null)
            //        continue;

            //    var original = card.FrenchText;

            //    card.FrenchText = card.FrenchText.Replace("oR", "(R)");
            //    card.FrenchText = card.FrenchText.Replace("oG", "(G)");
            //    card.FrenchText = card.FrenchText.Replace("oW", "(W)");
            //    card.FrenchText = card.FrenchText.Replace("oU", "(U)");
            //    card.FrenchText = card.FrenchText.Replace("oB", "(B)");
            //    card.FrenchText = card.FrenchText.Replace("ocT", "(T)");

            //    for (int id = 16; id >= 0; id--)
            //    {
            //        card.FrenchText = card.FrenchText.Replace("o" + id, "(" + id + ")");
            //    }

            //    if (card.FrenchText != original)
            //    {
            //        Debug.WriteLine("Updated {0} to {1}", original, card.FrenchText);
            //    }
            //}
            //DataRoot.Entities.SaveChanges();            
            //return;
            //var path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "err.txt");
            //File.Delete(path);

            //foreach (Card card in list)
            //{
            //    Debug.WriteLine("Processing {0}:{1}   {2}/{3}", card.Expansion.Name, card.Name, index, list.Count);
            //    if (card.ManaCost == null || card.ConvertedManaCost == null || card.FrenchMultiverseID == null)
            //    {
            //        card.FrenchName = null;
            //        card.FrenchFlavor = null;
            //        Tools.GetCardInfos(card);
            //        DataRoot.Entities.SaveChanges();
            //        Debug.WriteLine("Done: {0}:{1}/{2}", card.FrenchName, card.ManaCost, card.ConvertedManaCost);
            //    }
            //    index++;
            //}
        }

        public static void Backup(string filename)
        {
            if (File.Exists(filename))
                File.Delete(filename);

            using (Ionic.Zip.ZipFile zip = new Ionic.Zip.ZipFile())
            {
                zip.AddDirectory(DataRoot.WorkingPath);
                zip.Save(filename);
            }
        }

        public static void Restore(string filename)
        {
            if (!Directory.Exists(DataRoot.WorkingPath))
                Directory.CreateDirectory(DataRoot.WorkingPath);

            if (Directory.Exists(DataRoot.WorkingPath + "old"))
                Directory.Delete(DataRoot.WorkingPath + "old", true);

            Directory.Move(DataRoot.WorkingPath, DataRoot.WorkingPath + "old");

            Directory.CreateDirectory(DataRoot.WorkingPath);

            try
            {
                using (ZipInputStream s = new ZipInputStream(File.OpenRead(filename)))
                {
                    ZipEntry theEntry;
                    while ((theEntry = s.GetNextEntry()) != null)
                    {
                        if (theEntry.IsDirectory)
                        {
                            Directory.CreateDirectory(Path.Combine(DataRoot.WorkingPath, theEntry.Name));
                            continue;
                        }

                        using (FileStream streamWriter = File.Create(Path.Combine(DataRoot.WorkingPath, theEntry.Name)))
                        {
                            long size = theEntry.Size;
                            byte[] data = new byte[size];

                            while (true)
                            {
                                size = s.Read(data, 0, data.Length);
                                if (size > 0)
                                {
                                    streamWriter.Write(data, 0, (int)size);
                                }
                                else
                                {
                                    break;
                                }
                            }
                        }
                    }
                }
                Directory.Delete(DataRoot.WorkingPath + "old", true);
            }
            catch (Exception)
            {
                if (Directory.Exists(DataRoot.WorkingPath))
                    Directory.Delete(DataRoot.WorkingPath, true);
                Directory.Move(DataRoot.WorkingPath + "old", DataRoot.WorkingPath);
                throw;
            }
        }

        public void Commit()
        {
            DataRoot.Entities.SaveChanges();
        }

        public void Rollback()
        {
            foreach (var entity in DataRoot.Entities.ObjectStateManager.GetObjectStateEntries(EntityState.Added))
            {
                entity.ChangeState(EntityState.Detached);
            }
        }
    }
}
