﻿using System;
using System.Deployment.Application;
using System.IO;
using System.Windows;
using UrzaGatherer.ViewModels;
using UrzaIncubator;

namespace UrzaGatherer
{
    public partial class App
    {
        readonly RootViewModel rootViewModel;
        public App()
        {
            ShutdownMode = ShutdownMode.OnExplicitShutdown;

            InitializeComponent();

            rootViewModel = new RootViewModel();

            if (!RootViewModel.Prepare(ApplicationDeployment.IsNetworkDeployed))
            {
                ApplicationDeployment.CurrentDeployment.DownloadFileGroupCompleted += CurrentDeployment_DownloadFileGroupCompleted;
                ApplicationDeployment.CurrentDeployment.DownloadFileGroupProgressChanged += CurrentDeployment_DownloadFileGroupProgressChanged;
                ApplicationDeployment.CurrentDeployment.DownloadFileGroupAsync("SDF");

                ProgressWindow.Show("Downloading database");
            }

            Launch();
        }

        public RootViewModel RootViewModel
        {
            get { return rootViewModel; }
        }

        void CurrentDeployment_DownloadFileGroupProgressChanged(object sender, DeploymentProgressChangedEventArgs e)
        {
            Dispatcher.Invoke(new Action(() =>
                                             {
                                                 ProgressWindow.Progress = e.ProgressPercentage;
                                             }));
        }

        void CurrentDeployment_DownloadFileGroupCompleted(object sender, DownloadFileGroupCompletedEventArgs e)
        {
            Dispatcher.Invoke(new Action(() =>
                                             {
                                                 ProgressWindow.HideProgress();
                                                 File.Copy(Path.Combine(Environment.CurrentDirectory, "UrzaGatherer.sdf"), Path.Combine(DataRoot.WorkingPath, "UrzaGatherer.sdf"));
                                                 Launch();
                                             }));
        }

        void Launch()
        {
            SplashScreen splash = new SplashScreen("nicol.jpg");
            splash.Show(false);

            bool notOK = true;
            bool allowApplicationStart = true;

            while (notOK)
            {
                try
                {
                    RootViewModel.Start();
                    notOK = false;
                }
                catch (Exception)
                {
                    InstallDatabaseWindow databaseWindow = new InstallDatabaseWindow();

                    if (databaseWindow.ShowDialog() != true)
                    {
                        allowApplicationStart = false;
                        notOK = false;
                    }
                }
            }

            if (allowApplicationStart)
            {
                MainWindow mainWindow = new MainWindow();
                mainWindow.Closed += mainWindow_Closed;
                mainWindow.Show();
            }

            splash.Close(!allowApplicationStart ? TimeSpan.Zero : TimeSpan.FromSeconds(1));

            if (!allowApplicationStart)
                Shutdown();
        }

        void mainWindow_Closed(object sender, EventArgs e)
        {
            Shutdown();
        }
    }
}

