﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using UrzaGatherer.ViewModels;
using UrzaIncubator;

namespace UrzaGatherer
{
    public partial class BlocksTreeView
    {
        static readonly DependencyProperty SelectedItemProperty = DependencyProperty.Register("SelectedItem", typeof (object), typeof (BlocksTreeView));

        public BlocksTreeView()
        {
            InitializeComponent();
        }

        public object SelectedItem
        {
            get { return GetValue(SelectedItemProperty); }
            set { Utilities.SetSelected(blockTreeview, value); }
        }

        private void TextBlock_TargetUpdated(object sender, DataTransferEventArgs e)
        {
            TextBlock textBlock = (TextBlock)sender;

            BindingOperations.GetBindingExpressionBase(textBlock, TextBlock.TextProperty).UpdateTarget();
        }

        private void blockTreeview_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            ExpansionLogo.Opacity = 0;

            DoubleAnimation animation = new DoubleAnimation(0, 1, new Duration(TimeSpan.FromMilliseconds(1000)));
            ExpansionLogo.BeginAnimation(OpacityProperty, animation);

            SetValue(SelectedItemProperty, blockTreeview.SelectedItem);

            RootViewModel rootViewModel = (Application.Current as App).RootViewModel;
            rootViewModel.ExpansionViewModel.Current = blockTreeview.SelectedItem as Expansion;
            rootViewModel.BlockViewModel.Current = blockTreeview.SelectedItem as Block;
        }

        private void blockTreeview_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void blockTreeview_DragOver(object sender, DragEventArgs e)
        {
            Point position = e.GetPosition(blockTreeview);

            HitTestResult result = VisualTreeHelper.HitTest(blockTreeview, position);

            if (result.VisualHit != null)
            {
                FrameworkElement element = (FrameworkElement)result.VisualHit;

                Utilities.SetSelected(blockTreeview, element.DataContext);
            }
        }

        private void blockTreeview_Drop(object sender, DragEventArgs e)
        {
            try
            {
                string[] datas = (string[])e.Data.GetData("FileName");
                Block block = blockTreeview.SelectedItem as Block;

                if (block == null)
                {
                    Expansion expansion = blockTreeview.SelectedItem as Expansion;

                    if (Keyboard.Modifiers == ModifierKeys.Control)
                        expansion.Logo = File.ReadAllBytes(datas[0]);
                    else
                        expansion.Image = File.ReadAllBytes(datas[0]);
                }
                else
                {
                    block.Image = File.ReadAllBytes(datas[0]);
                }
            }
            catch (Exception)
            {
                MessageWindow.Show("Unable to update image.", "Urza Gatherer", null);
            }
        }
    }
}
