﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media.Animation;
using UrzaIncubator;
using System.ComponentModel;

namespace UrzaGatherer
{
    public partial class CardEditor : INotifyPropertyChanged
    {
        bool isCollapsed = true;
        public event EventHandler Closed;       

        public CardEditor()
        {
            InitializeComponent();
        }

        public bool IsCollapsed
        {
            get { return isCollapsed; }
            set 
            { 
                isCollapsed = value; 

                if (PropertyChanged != null)
                    PropertyChanged(this, new PropertyChangedEventArgs("IsCollapsed"));
            }
        }

        private void rootGrid_Drop(object sender, DragEventArgs e)
        {
            Utilities.ChangeImage((Card)((CollectionViewSource)DataContext).View.CurrentItem, e);
            e.Handled = true;
        }

        private void rootGrid_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Escape)
                CloseCardEditor(false);
            else if (e.Key == Key.Enter)
            {
                EditorCloseButton.Focus();
                CloseCardEditor(true);
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            CloseCardEditor(true);
        }

        void CloseCardEditor(bool save)
        {
            (FindResource("FlowUpStoryboard") as Storyboard).Begin();
            if (save)
            {
                Card card = (DataContext as CollectionViewSource).View.CurrentItem as Card;

                card.ForceRefresh();
            }

            IsCollapsed = true;

            if (Closed != null)
                Closed(this, EventArgs.Empty);
        }

        private void Image_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void Image_Drop(object sender, DragEventArgs e)
        {
            Utilities.ChangeImage((Card)((CollectionViewSource)DataContext).View.CurrentItem, e);
            e.Handled = true;
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            RarityList.SelectedIndex = 0;
            AuthorsList.DisplayMemberPath = "Name";
        }

        internal void Show()
        {
            (FindResource("FlowDownStoryboard") as Storyboard).Begin();

            IsCollapsed = false;
        }

        private void rootGrid_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
