﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using UrzaGatherer.ViewModels;
using UrzaIncubator;

namespace UrzaGatherer
{
    public partial class CardsList
    {
        static readonly DependencyProperty SelectedItemProperty = DependencyProperty.Register("SelectedItem", typeof(object), typeof(CardsList));

        public CardsList()
        {
            InitializeComponent();
        }

        public object SelectedItem
        {
            get { return GetValue(SelectedItemProperty); }
            set { Utilities.SetSelected(cardsGrid, value); }
        }

        public int SelectedIndex
        {
            get { return cardsGrid.SelectedIndex; }
            set { cardsGrid.SelectedIndex = value; }
        }

        private void cardsGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            cardsGrid.ScrollIntoView(cardsGrid.SelectedItem);

            if (cardsGrid.SelectedItem != null)
            {
                Card card = (Card)cardsGrid.SelectedItem;
                Clipboard.SetText(card.Name);

                SetValue(SelectedItemProperty, card);

                ListBoxItem item = (ListBoxItem)cardsGrid.ItemContainerGenerator.ContainerFromItem(card);
                if (item != null)
                    item.Focus();

            }
        }

        void MoveCursorToTextBlock(TextBlock textBlock)
        {
            int column = Grid.GetColumn(textBlock);
            MoveCursorToColumn(column);
        }

        void MoveCursorToColumn(int column)
        {
            ExpansionViewModel expansionViewModel = DataContext as ExpansionViewModel;
            SortCursor.Opacity = 1.0;
            double x = 0;
            for (int index = 0; index < column; index++)
            {
                x += HeaderGrid.ColumnDefinitions[index].ActualWidth;
            }

            SortCursorTransform.X = x + HeaderGrid.ColumnDefinitions[column].ActualWidth / 2 - 5;
            SortCursorTransform.Y = expansionViewModel.SortDirection == ListSortDirection.Ascending ? -20 : 22;

            SortCursorRotation.Angle = expansionViewModel.SortDirection == ListSortDirection.Ascending ? 180 : 0;
        }

        private void TextBlock_MouseDown(object sender, MouseButtonEventArgs e)
        {
            TextBlock textBlock = (TextBlock)sender;
            ExpansionViewModel expansionViewModel = DataContext as ExpansionViewModel;

            string oldOrder = expansionViewModel.OrderBy;

            switch (textBlock.Text)
            {
                case "N°":
                    expansionViewModel.OrderBy = "Number";
                    break;
                case "Name":
                    expansionViewModel.OrderBy = "Name";
                    break;
                case "Color":
                    expansionViewModel.OrderBy = "Color.Name";
                    break;
                case "Rarity":
                    expansionViewModel.OrderBy = "Rarity.Name";
                    break;
                case "Author":
                    expansionViewModel.OrderBy = "Author.Name";
                    break;
                case "Exp.":
                    expansionViewModel.OrderBy = "Expansion.Name";
                    break;
                case "Force":
                    expansionViewModel.OrderBy = "Force";
                    break;
                case "Def.":
                    expansionViewModel.OrderBy = "Defense";
                    break;
                case "Price":
                    expansionViewModel.OrderBy = "Price";
                    break;
            }

            if (oldOrder != expansionViewModel.OrderBy)
                expansionViewModel.SortDirection = ListSortDirection.Ascending;
            else if (expansionViewModel.SortDirection == ListSortDirection.Ascending)
                expansionViewModel.SortDirection = ListSortDirection.Descending;
            else
                expansionViewModel.SortDirection = ListSortDirection.Ascending;

            expansionViewModel.Refresh();

            MoveCursorToTextBlock(textBlock);
        }


        private void cardsGrid_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void cardsGrid_DragOver(object sender, DragEventArgs e)
        {
            Point position = e.GetPosition(cardsGrid);

            HitTestResult result = VisualTreeHelper.HitTest(cardsGrid, position);

            if (result.VisualHit != null)
            {
                FrameworkElement element = (FrameworkElement)result.VisualHit;
                cardsGrid.SelectedItem = element.DataContext;
            }
        }

        private void cardsGrid_Drop(object sender, DragEventArgs e)
        {
            Utilities.ChangeImage((Card)cardsGrid.SelectedItem, e);
        }

        private void cardsGrid_KeyDown(object sender, KeyEventArgs e)
        {
            if (cardsGrid.SelectedItem == null)
                return;

            if (e.Key == Key.Enter)
            {
                Card card = (Card)cardsGrid.SelectedItem;
                card.Check = !card.Check;
                cardsGrid.SelectedIndex++;
                UpdateCounters();
            }
        }

        private void UpdateCounters()
        {
            ExpansionViewModel expansionViewModel = DataContext as ExpansionViewModel;

            expansionViewModel.RootViewModel.Commit();
            ((App)Application.Current).RootViewModel.BlockViewModel.RecomputeCounters();
        }

        private void GridSplitter_DragDelta(object sender, System.Windows.Controls.Primitives.DragDeltaEventArgs e)
        {
            GridSplitter splitter = (GridSplitter)sender;
            int column = (int)splitter.GetValue(Grid.ColumnProperty);

            MoveCursorToColumn(column);
        }

        private void CheckBox_SourceUpdated(object sender, DataTransferEventArgs e)
        {
            UpdateCounters();
        }

        private void Grid_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount != 2)
                return;

            (Application.Current as App).RootViewModel.ExpansionViewModel.EditCurrentCard();
        }
    }
}
