﻿using System;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Linq;
using UrzaIncubator;

namespace UrzaGatherer
{
    public partial class ImagesList
    {
        readonly DependencyProperty ImagesScaleProperty = DependencyProperty.Register("ImagesScale", typeof (double), typeof (ImagesList));
        public event Action<Card> CardWithoutPictureDetected;
        public event Action<Card> SelectedCardChanged;

        public double ImagesScale
        {
            get { return (double)GetValue(ImagesScaleProperty); }
            set
            {
                SetValue(ImagesScaleProperty, value);
            }
        }

        public ImagesList()
        {
            InitializeComponent();
        }

        private void imagesList_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void imagesList_DragOver(object sender, DragEventArgs e)
        {
            Point position = e.GetPosition(imagesList);

            HitTestResult result = VisualTreeHelper.HitTest(imagesList, position);

            if (result.VisualHit != null)
            {
                FrameworkElement element = (FrameworkElement)result.VisualHit;
                imagesList.SelectedItem = element.DataContext;
            }
        }

        private void imagesList_Drop(object sender, DragEventArgs e)
        {
            Utilities.ChangeImage((Card)imagesList.SelectedItem, e);
        }

        private void imagesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Card card = (Card)imagesList.SelectedItem;
            if (card != null && (card.CompletePath == null || card.Type == null || card.Text == null))
            {
                if (CardWithoutPictureDetected != null)
                    CardWithoutPictureDetected(card);
            }
            imagesList.ScrollIntoView(imagesList.SelectedItem);

            if (card != null && card.Bitmap != null)
            {
                if (SelectedCardChanged != null)
                    SelectedCardChanged(card);
            }
        }

        private void cardImage_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount != 2)
                return;
            (Application.Current as App).RootViewModel.ExpansionViewModel.EditCurrentCard();
        }

        public void DelayedScrollToCurrent()
        {
            Task.Factory.StartNew(() =>
            {
                Thread.Sleep(500);
                Dispatcher.Invoke(new Action(() => imagesList.ScrollIntoView(imagesList.SelectedItem)));
            });
        }
    }
}
