﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.IO;
using Ookii.Dialogs.Wpf;
using UrzaIncubator;

namespace UrzaGatherer
{
    /// <summary>
    /// Interaction logic for ImportPicturesWindow.xaml
    /// </summary>
    public partial class ImportPicturesWindow
    {
        Expansion expansion;

        public Expansion Expansion
        {
            set { expansion = value; }
        }

        public ImportPicturesWindow()
        {
            InitializeComponent();
        }

        private void BrowseButton_Click(object sender, RoutedEventArgs e)
        {
            VistaFolderBrowserDialog dialog = new VistaFolderBrowserDialog();
            dialog.Description = @"Please select a folder containing pictures for the expansion.";
            dialog.ShowNewFolderButton = false;
            dialog.UseDescriptionForTitle = true;
            if (dialog.ShowDialog() == true)
            {
                TextPath.Text = dialog.SelectedPath;
                DoImport();
            }
        }

        private void DoImport()
        {
            string path = TextPath.Text;
            Progress.Maximum = expansion.Cards.Count();
            Progress.Value = 0;
            CloseButton.IsEnabled = false;
            CardsList.Items.Clear();
            ImagesList.Items.Clear();
            Task.Factory.StartNew(() =>
                {
                    DirectoryInfo info = new DirectoryInfo(path);

                    foreach (Card card in expansion.Cards)
                    {
                        Dispatcher.Invoke(new Action(() =>
                        {
                            Progress.Value++;
                            TextReport.Text = string.Format("{0:0.##} %", (Progress.Value * 100.0) / Progress.Maximum);
                        }));
                        bool find = false;
                        foreach (FileInfo file in info.GetFiles())
                        {
                            string cardName = card.Name.ToLower().Replace(")", "").Replace("(ver.", "").Replace("(v.", "").Replace("’", "'").Replace("//", "][").Replace(" ", "").Replace("æ", "ae").Replace(":", "");
                            if (file.Name.ToLower().Replace(" ", "").Contains(cardName))
                            {
                                card.Image = File.ReadAllBytes(file.FullName);
                                find = true;
                                break;
                            }
                        }

                        if (!find)
                        {
                            Dispatcher.Invoke(new Action(() =>
                                {
                                    CardsList.Items.Add(card);
                                }));
                        }
                    }

                    Dispatcher.Invoke(new Action(() =>
                        {
                            foreach (FileInfo file in info.GetFiles())
                            {

                                ImagesList.Items.Add(file);

                            }
                            CloseButton.IsEnabled = true;
                        }));
                });
        }

        private void AffectButton_Click(object sender, RoutedEventArgs e)
        {
            if (CardsList.SelectedItem == null)
                return;
            if (ImagesList.SelectedItem == null)
                return;

            Card card = CardsList.SelectedItem as Card;
            string path = ImagesList.SelectedItem.ToString();

            card.Image = File.ReadAllBytes(Path.Combine(TextPath.Text, path));

            CardsList.Items.Remove(card);
            ImagesList.Items.RemoveAt(ImagesList.SelectedIndex);
        }

        private void ImagesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (ImagesList.SelectedItem == null)
            {
                ImagePreview.Source = null;
                return;
            }
            string path = ImagesList.SelectedItem.ToString();
            byte[] bytes = File.ReadAllBytes((Path.Combine(TextPath.Text, path)));

            ImageSourceConverter converter = new ImageSourceConverter();

            ImagePreview.Source = (ImageSource)converter.ConvertFrom(bytes);
        }

        private void Close_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }
    }
}
