﻿using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Shell;
using System.Windows.Threading;
using UrzaIncubator;
using System.Windows;

namespace UrzaGatherer
{
    partial class MainWindow
    {
        DateTime pricesProcessStart;

        void UpdateProgess(int index)
        {
            Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(() =>
                {
                    TimeSpan diff = DateTime.Now - pricesProcessStart;
                    double current = pricesProgressBar.Value;
                    double remaining = pricesProgressBar.Maximum - current;
                    double ellapsed = diff.TotalSeconds;
                    long remainingSeconds = (long)(ellapsed * remaining / current);
                    long remainingHours = remainingSeconds / 3600;
                    remainingSeconds = remainingSeconds - 3600 * remainingHours;

                    long remainingMinutes = remainingSeconds / 60;
                    remainingSeconds = remainingSeconds - 60 * remainingMinutes;

                    pricesProgressText.ToolTip = string.Format("{0} hour(s) {1} minute(s) {2} second(s) remaining", remainingHours, remainingMinutes, remainingSeconds);
                    mainTaskBarItem.ProgressValue = pricesProgressBar.Value / pricesProgressBar.Maximum;
                    pricesProgressBar.Value += index;
                }));
        }

        void GetExpansionPrice(Expansion expansion)
        {
            pricesProgressText.Text = "Retreiving prices of " + expansion.Name;
            pricesProgressText.ToolTip = pricesProgressText.Text;
            pricesProgressBar.Maximum = expansion.Cards.Count();

            Task.Factory.StartNew(() =>
            {
                PriceManager.GetExpansionPrice(expansion, UpdateProgess);
                EndPricesProcess();
            });
        }

        private void GetPriceButton_Click(object sender, RoutedEventArgs e)
        {
            if (QuestionWindow.Show("Are you sure you want to get averages prices from internet?\nThe process can take several hours.", "Confirmation", this) != true)
            {
                return;
            }

            pricesProcessStart = DateTime.Now;

            pricesProgressText.Visibility = Visibility.Visible;
            GetPriceButton.IsEnabled = false;
            pricesProgressBar.Visibility = Visibility.Visible;
            pricesProgressBar.Value = 0;
            pricesProgressBar.Minimum = 0;

            if (blockTreeview.SelectedItem != null && blockTreeview.SelectedItem is Expansion)
            {
                GetExpansionPrice(blockTreeview.SelectedItem as Expansion);
                return;
            }
            Block rootBlock = null;
            if (blockTreeview.SelectedItem != null && blockTreeview.SelectedItem is Block)
            {
                rootBlock = blockTreeview.SelectedItem as Block;
            }

            pricesProgressText.Text = "";
            pricesProgressText.ToolTip = "";
            pricesProgressText.Text = "Processing prices";
            pricesProgressText.ToolTip = pricesProgressText.Text;

            mainTaskBarItem.ProgressState = TaskbarItemProgressState.Normal;

            Task.Factory.StartNew(() =>
            {
                Expansion[] expansions = null;

                if (rootBlock != null)
                {
                    Dispatcher.Invoke(DispatcherPriority.ContextIdle, new Action(() =>
                                                                                     {
                                                                                         expansions = rootBlock.Expansions.ToArray();
                                                                                     }));
                }
                else
                {
                    expansions = DataRoot.Entities.Expansions.ToArray();
                }

                Dispatcher.Invoke(DispatcherPriority.ContextIdle, new Action(() => pricesProgressBar.Maximum = expansions.Select(exp => exp.Cards.Count()).Sum()));

                Semaphore semaphore = new Semaphore(2, 2);
                Parallel.For(0, expansions.Length, index =>
                {
                    semaphore.WaitOne();
                    Expansion expansion = expansions[index];
                    PriceManager.GetExpansionPrice(expansion, UpdateProgess);
                    semaphore.Release();
                });

                EndPricesProcess();
            }, TaskCreationOptions.LongRunning);
        }

        void EndPricesProcess()
        {
            Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(() =>
            {
                pricesProgressBar.Visibility = Visibility.Collapsed;
                pricesProgressText.Visibility = Visibility.Collapsed;
                mainTaskBarItem.ProgressState = TaskbarItemProgressState.None;
                MessageWindow.Show("Prices retreived successfully.", "Urza Gatherer", this);
                GetPriceButton.IsEnabled = true;
            }));
        }
    }
}
