﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Threading;
using System.Xml;
using Ookii.Dialogs.Wpf;
using UrzaIncubator;
using Application = System.Windows.Application;
using Clipboard = System.Windows.Clipboard;
using Color = UrzaIncubator.Color;
using DragDropEffects = System.Windows.DragDropEffects;
using DragEventArgs = System.Windows.DragEventArgs;
using MessageBox = System.Windows.MessageBox;
using Point = System.Windows.Point;


namespace UrzaGatherer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        string orderBy = "Number";
        ListSortDirection sortDirection = ListSortDirection.Ascending;
        readonly ManualResetEvent resetEvent = new ManualResetEvent(false);
        readonly ManualResetEvent queueEvent = new ManualResetEvent(false);
        readonly Queue<Card> imagesSearchingCards = new Queue<Card>();
        bool loadedDone;
        readonly Stack<IEnumerable<Card>> requestStack = new Stack<IEnumerable<Card>>();
        IEnumerable<Card> currentQuery;
        Task currentQueryTask;
        bool itemAdded;
        CancellationTokenSource queryCancellation;
        int currentChecked;
        int currentMissing;
        float currentValueCount;
        float totalValueCount;

        public MainWindow()
        {
            string path = Path.Combine(System.Windows.Forms.Application.StartupPath, "UrzaGatherer.xml");

            InitializeComponent();

            MainHeader.DataContext = new[] { "Home", "Tools", "Database" };
            RightHeader.DataContext = new[] { "Images", "Statistics", "Filters" };

            if (File.Exists(path))
            {
                XmlTextReader reader = new XmlTextReader(path);

                reader.ReadToFollowing("config");
                WindowState = (WindowState)Enum.Parse(typeof(WindowState), reader.GetAttribute("windowState"));

                ScaleSlider.Value = double.Parse(reader.GetAttribute("ImageScale"));

                if (reader.GetAttribute("splitter3") != null)
                {
                    MainColumn01.Width = new GridLength(double.Parse(reader.GetAttribute("splitter1")), GridUnitType.Star);
                    MainColumn02.Width = new GridLength(double.Parse(reader.GetAttribute("splitter2")), GridUnitType.Star);
                    MainColumn03.Width = new GridLength(double.Parse(reader.GetAttribute("splitter3")), GridUnitType.Star);
                }

                reader.Close();
            }

            Task.Factory.StartNew(ThreadJob, TaskCreationOptions.LongRunning);
            Task.Factory.StartNew(ProcessRequestJob, TaskCreationOptions.LongRunning);

            DoCharting();

            ImagesSlideControl.TransitionCompleted += ImagesSlideControl_TransitionCompleted;
        }

        void ImagesSlideControl_TransitionCompleted(object sender, EventArgs e)
        {
            Task.Factory.StartNew(() =>
                                             {
                                                 Thread.Sleep(500);
                                                 Dispatcher.Invoke(new Action(() => imagesList.ScrollIntoView(imagesList.SelectedItem)));
                                             });

        }

        void ThreadJob()
        {
            do
            {
                resetEvent.WaitOne();

                while (imagesSearchingCards.Count > 0)
                {
                    Card card;
                    lock (imagesSearchingCards)
                    {
                        card = imagesSearchingCards.Dequeue();
                        resetEvent.Reset();
                    }

                    if (card.CompletePath == null)
                        Tools.GetCardImage2(card, "en");
                    if (card.Type == null || card.Text == null)
                        Tools.GetCardInfos(card);
                }

            }
            while (true);
        }

        static void GetExpansionsAndBlocksCount(IEnumerable<Card> list, int expansionID, int blockID, out int expansionsCount, out int blocksCount)
        {
            if (list == null)
            {
                var query = from card in DataRoot.Entities.Cards
                            where (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                            select card.ExpansionID;
                expansionsCount = query.Distinct().Count();

                var query2 = from card in DataRoot.Entities.Cards
                             where (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                             select card.Expansion.BlockID;
                blocksCount = query2.Distinct().Count();
            }
            else
            {
                var query = from card in list
                            where (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                            select card.ExpansionID;
                expansionsCount = query.Distinct().Count();

                var query2 = from card in list
                             where (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                             select card.Expansion.BlockID;
                blocksCount = query2.Distinct().Count();
            }
        }

        static int GetResultCount(IEnumerable<Card> list, int colorID, int rarityID, int expansionID, int blockID, bool? check)
        {
            int result;
            if (list == null)
            {
                var query = from card in DataRoot.Entities.Cards
                            where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                            && (!check.HasValue || card.Check == check.Value)
                            select card;
                result = query.Count();
            }
            else
            {
                var query = from card in list
                            where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                            && (!check.HasValue || card.Check == check.Value)
                            select card;
                result = query.Count();
            }

            return result;
        }

        static float GetResultValue(IEnumerable<Card> list, int colorID, int rarityID, int expansionID, int blockID, bool? check)
        {
            try
            {
                if (list == null)
                {
                    var query = from card in DataRoot.Entities.Cards
                                where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                                && (!check.HasValue || card.Check == check.Value)
                                select card.Price.HasValue ? card.Price.Value : 0;
                    return (float)query.Sum();
                }
                else
                {
                    var query = from card in list
                                where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID == 0 || card.ExpansionID == expansionID) && (blockID == 0 || card.Expansion.BlockID == blockID)
                                && (!check.HasValue || card.Check == check.Value)
                                select card.Price.HasValue ? card.Price.Value : 0;
                    return (float)query.Sum();
                }
            }
            catch
            {
                return 0;
            }
        }

        void DoCharting(IEnumerable<Card> list = null)
        {
            if (ColorsDistribution == null)
                return;

            Expansion expansion = null;
            Block block = null;

            Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                                                                           {
                                                                               expansion = blockTreeview.SelectedItem as Expansion;
                                                                               block = blockTreeview.SelectedItem as Block;
                                                                           }));

            List<DataChart> colorsCount = new List<DataChart>();
            int expansionID = 0;
            int blockID = 0;

            if (expansion != null)
                expansionID = expansion.ID;

            if (block != null && block.Name != "Oracle")
                blockID = block.ID;

            int globalCount = GetResultCount(list, 0, 0, expansionID, blockID, null);

            // Colors
            foreach (Color color in DataRoot.Entities.Colors)
            {
                if (color.Name != " ")
                {
                    int colorID = color.ID;
                    int result = GetResultCount(list, colorID, 0, expansionID, blockID, null);

                    colorsCount.Add(new DataChart { Value = result, Name = color.Name, Binding = color });
                }
            }

            // Rarity
            List<DataChart> rarityCount = new List<DataChart>();

            foreach (Rarity rarity in DataRoot.Entities.Rarities)
            {
                if (rarity.Name != " ")
                {
                    int rarityID = rarity.ID;
                    int result = GetResultCount(list, 0, rarityID, expansionID, blockID, null);

                    rarityCount.Add(new DataChart { Value = result, Name = rarity.Name });
                }
            }

            // Expansions & blocks
            int expansionsCount;
            int blocksCount;

            GetExpansionsAndBlocksCount(list, expansionID, blockID, out expansionsCount, out blocksCount);

            // Complete
            int complete = GetResultCount(list, 0, 0, expansionID, blockID, true);

            currentValueCount = GetResultValue(list, 0, 0, expansionID, blockID, true);
            totalValueCount = GetResultValue(list, 0, 0, expansionID, blockID, null);

            Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                                                                            {
                                                                                ColorsDistribution.DataContext = colorsCount;
                                                                                RarityDistribution.DataContext = rarityCount;

                                                                                // Total
                                                                                StatsCardCount.Text = globalCount.ToString();

                                                                                // Complete
                                                                                currentMissing = globalCount - complete;
                                                                                currentChecked = complete;
                                                                                ShowCompleteLevel();

                                                                                // Expansions and Blocks
                                                                                ExpansionsCount.Text = expansionsCount.ToString();
                                                                                BlocksCount.Text = blocksCount.ToString();
                                                                            }));
        }

        void ShowCompleteLevel()
        {
            int globalCount = currentChecked + currentMissing;

            CompleteCardCount.Text = string.Format("{0:0.00}%", ((currentChecked * 100.0f) / globalCount));
            CheckedCount.Text = currentChecked.ToString();
            MissingCount.Text = currentMissing.ToString();

            // Value
            OwnValueCount.Text = string.Format("{0:0.00} €", currentValueCount);
            ValueCount.Text = string.Format("{0:0.00} €", totalValueCount);
        }

        private void CommandBinding_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }

        private void CheckBox_SourceUpdated(object sender, DataTransferEventArgs e)
        {
            DataRoot.Entities.SaveChanges();
            UpdateCurrentCompleteLevel((sender as FrameworkElement).DataContext as Card);

            SynchroniseTreeview();
        }

        private void UpdateCurrentCompleteLevel(Card card)
        {
            if (card.Check)
            {
                currentChecked++;
                currentMissing--;

                if (card.Price.HasValue)
                {
                    currentValueCount += (float)card.Price;
                }
            }
            else
            {
                currentChecked--;
                currentMissing++;

                if (card.Price.HasValue)
                {
                    currentValueCount -= (float)card.Price;
                }
            }
        }

        void SynchroniseTreeview()
        {
            Dispatcher.Invoke(new Action(() =>
                                             {
                                                 Expansion expansion = ((Card)cardsGrid.SelectedItem).Expansion;
                                                 expansion.Complete = expansion.Cards.All(card => card.Check);

                                                 Block block = expansion.Block;
                                                 block.Complete = block.Expansions.All(exp => exp.Complete);

                                                 if (checkFilters.SelectedIndex > 0)
                                                     RefreshCardsGrid();

                                                 ShowCompleteLevel();
                                             }));
        }

        private void TextBlock_TargetUpdated(object sender, DataTransferEventArgs e)
        {
            TextBlock textBlock = (TextBlock)sender;

            BindingOperations.GetBindingExpressionBase(textBlock, TextBlock.TextProperty).UpdateTarget();
        }

        private void blockTreeview_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            ExpansionLogo.Opacity = 0;

            DoubleAnimation animation = new DoubleAnimation(0, 1, new Duration(TimeSpan.FromMilliseconds(1000)));
            ExpansionLogo.BeginAnimation(OpacityProperty, animation);

            RefreshCardsGrid();
            if (SortCursor.Opacity == 0)
                MoveCursorToTextBlock(numberColumHeader);
        }

        void RefreshCardsGrid()
        {
            if (!loadedDone)
                return;
            ResetState();

            Block block = blockTreeview.SelectedItem as Block;

            if (block != null)
            {
                if (block.Name == "Oracle")
                {
                    DoOracleForecast();
                    return;
                }
            }

            Expansion expansion = blockTreeview.SelectedItem as Expansion;
            if (expansion == null)
            {
                EnqueueQuery(null);
                return;
            }

            IEnumerable<Card> query = BuildQuery(expansion);

            EnqueueQuery(query);
        }

        IEnumerable<Card> BuildQuery(Expansion expansion)
        {
            bool dontUpdate;
            return GetQuery(out dontUpdate, expansion.ID);
        }

        void DoOracleForecast()
        {
            TooManyResults.Visibility = Visibility.Collapsed;

            bool dontUpdate;
            IQueryable<Card> query = GetQuery(out dontUpdate);

            if (dontUpdate)
            {
                TooManyResults.Visibility = Visibility.Visible;
                EnqueueQuery(null);
                return;
            }

            EnqueueQuery(query);
        }

        IQueryable<Card> GetQuery(out bool dontUpdate, int expansionID = 0)
        {
            bool? checkFilter = null;

            if (checkFilters.SelectedIndex == 1)
                checkFilter = true;
            else if (checkFilters.SelectedIndex == 2)
                checkFilter = false;

            string filterColor = ((Color)colorFilters.SelectedItem).Name;
            string filterRarity = ((Rarity)rarityFilters.SelectedItem).Name;
            string filterAuthor = ((Author)authorFilters.SelectedItem).Name;
            string filterName = nameFilters.Text.ToLower();
            string filterFlavor = flavorFilters.Text.ToLower();
            string filterType = typeFilters.Text.ToLower();
            string filterFullText = fullTextFilters.Text.ToLower();
            string filterForceText = forceFilters.Text;
            string filterDefenseText = defenseFilters.Text;
            string filterPriceText = priceFilters.Text;
            string filterText = textFilters.Text;
            int forceFilter;
            int defenseFilter;
            float priceFilter;

            int.TryParse(filterForceText, out forceFilter);
            int.TryParse(filterDefenseText, out defenseFilter);
            float.TryParse(filterPriceText, out priceFilter);

            bool enableColorFilter = filterColor != " ";
            bool enableRarityFilter = filterRarity != " ";
            bool enableAuthorFilter = filterAuthor != " ";
            bool enableNameFilter = filterName != "";
            bool enableFlavorFilter = filterFlavor != "";
            bool enableTextFilter = filterText != "";
            bool enableTypeFilter = filterType != "";
            bool enableFullFilter = filterFullText != "";

            dontUpdate = (filterColor == " " && filterRarity == " " && filterAuthor == " " && (filterName.Length < 3) && (filterText.Length < 3) && (filterFlavor.Length < 3) && (filterType.Length < 3) && (filterFullText.Length < 3) && forceFilter < 2 && defenseFilter < 2 && priceFilter == 0);

            return from card in DataRoot.Entities.Cards
                   where
                       (expansionID <= 0 || card.ExpansionID == expansionID)
                       &&
                       (card.Check == checkFilter || !checkFilter.HasValue)
                       &&
                       (card.Color.Name == filterColor || !enableColorFilter)
                       &&
                       (card.Rarity.Name == filterRarity || !enableRarityFilter)
                       &&
                       (card.Author.Name == filterAuthor || !enableAuthorFilter)
                       &&
                       (forceFilter < 2 || card.Force >= forceFilter)
                       &&
                       (defenseFilter < 2 || card.Defense >= defenseFilter)
                       &&
                       (priceFilter == 0 || card.Price >= priceFilter)
                       &&
                       (!enableNameFilter || card.Name.Contains(filterName))
                       &&
                       (!enableTextFilter || card.Text.Contains(filterText))
                       &&
                       (!enableFlavorFilter || card.Flavor.Contains(filterFlavor))
                       &&
                       (!enableTypeFilter || card.Type.Contains(filterType))
                       &&
                       (!enableFullFilter || card.Type.Contains(filterFullText) || card.Flavor.Contains(filterFullText) || card.Name.Contains(filterFullText) || card.Text.Contains(filterFullText))
                   select card;
        }

        void EnqueueQuery(IEnumerable<Card> query)
        {
            lock (requestStack)
            {
                requestStack.Push(query);
                queueEvent.Set();
            }
        }

        void ProcessRequestJob()
        {
            while (true)
            {
                queueEvent.WaitOne();
                queueEvent.Reset();

                IEnumerable<Card> query = null;
                lock (requestStack)
                {
                    if (requestStack.Count > 0)
                        query = requestStack.Pop();
                    requestStack.Clear();
                }

                WaitForCurrentQueryToFinish();

                if (query == null)
                {
                    Dispatcher.Invoke(DispatcherPriority.Render, new Action(ResetState));
                    DoCharting();
                }
                else
                {
                    currentQuery = query;
                    DispatchListBinding(query);
                }
            }
        }

        void ResetState()
        {
            Block block = blockTreeview.SelectedItem as Block;

            if (block == null || block.Name != "Oracle")
                TooManyResults.Visibility = Visibility.Collapsed;
            CardsCount.Text = "";
            imagesList.DataContext = null;
            cardsGrid.DataContext = null;
            FrenchPart.DataContext = null;
            TextLoading.Visibility = Visibility.Collapsed;
        }

        void WaitForCurrentQueryToFinish()
        {
            if (currentQuery != null)
            {
                currentQuery = null;
                queryCancellation.Cancel();

                try
                {
                    currentQueryTask.Wait();
                }
                catch
                { }
            }
        }

        void DispatchListBinding(IEnumerable<Card> query)
        {
            CollectionViewSource source = null;
            Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                                    {
                                        source = new CollectionViewSource();
                                        TextLoading.Visibility = Visibility.Visible;
                                    }));

            queryCancellation = new CancellationTokenSource();
            CancellationToken token = queryCancellation.Token;
            currentQueryTask = Task.Factory.StartNew(() =>
                {
                    try
                    {
                        lock (DataRoot.Entities)
                        {
                            List<Card> list = query.ToList();

                            if (token.IsCancellationRequested)
                            {
                                return;
                            }

                            list.ForEach(c=>c.BindablePath = "Back.jpg");

                            Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                                {
                                    CardsCount.Text = string.Format("{0} card(s)", list.Count);

                                    source.Source = list;
                                    source.SortDescriptions.Clear();
                                    source.SortDescriptions.Add(new SortDescription(orderBy, sortDirection));

                                    if (token.IsCancellationRequested)
                                    {
                                        return;
                                    }

                                    cardsGrid.DataContext = source;
                                    imagesList.DataContext = source;

                                    cardsGrid.SelectedIndex = 0;

                                    if (token.IsCancellationRequested)
                                    {
                                        return;
                                    }
                                }));

                            DoCharting(list);

                            foreach (Card card in list)
                            {
                                if (token.IsCancellationRequested)
                                {
                                    return;
                                }

                                Card temp = card;
                                Dispatcher.Invoke(DispatcherPriority.Background, new Action(temp.BindPath));
                                Thread.Sleep(15);
                            }

                            Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                            {
                                if (token.IsCancellationRequested)
                                {
                                    return;
                                }

                                TextLoading.Visibility = Visibility.Collapsed;
                            }));

                            currentQuery = null;
                        }
                    }
                    catch
                    {

                    }
                }, token);
        }

        private void checkFilters_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void colorFilters_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void rarityFilters_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void authorFilters_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void Window_Closing(object sender, CancelEventArgs e)
        {
            bool canLeave;
            do
            {
                try
                {
                    DataRoot.Entities.SaveChanges();
                    canLeave = true;
                }
                catch (Exception)
                {
                    canLeave = false;
                    Thread.Sleep(100);
                }
            }
            while (!canLeave);


            XmlTextWriter writer = new XmlTextWriter(Path.Combine(System.Windows.Forms.Application.StartupPath, "UrzaGatherer.xml"), null);

            writer.WriteStartDocument();
            writer.WriteStartElement("config");

            writer.WriteStartAttribute("ImageScale");
            writer.WriteValue((int)ScaleSlider.Value);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("splitter1");
            writer.WriteValue(MainColumn01.ActualWidth);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("splitter2");
            writer.WriteValue(MainColumn02.ActualWidth);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("splitter3");
            writer.WriteValue(MainColumn03.ActualWidth);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("windowState");
            writer.WriteValue(WindowState.ToString());
            writer.WriteEndAttribute();

            writer.WriteEndElement();
            writer.WriteEndDocument();

            writer.Close();
        }

        private void cardsGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            cardsGrid.ScrollIntoView(cardsGrid.SelectedItem);

            if (cardsGrid.SelectedItem != null)
            {
                Card card = (Card)cardsGrid.SelectedItem;
                Clipboard.SetText(card.Name);

                FrenchPart.DataContext = card;
            }
        }

        void MoveCursorToTextBlock(TextBlock textBlock)
        {
            int column = Grid.GetColumn(textBlock);
            MoveCursorToColumn(column);
        }

        void MoveCursorToColumn(int column)
        {
            SortCursor.Opacity = 1.0;
            double x = 0;
            for (int index = 0; index < column; index++)
            {
                x += HeaderGrid.ColumnDefinitions[index].ActualWidth;
            }

            SortCursorTransform.X = x + HeaderGrid.ColumnDefinitions[column].ActualWidth / 2 - 5;
            SortCursorTransform.Y = sortDirection == ListSortDirection.Ascending ? -20 : 22;

            SortCursorRotation.Angle = sortDirection == ListSortDirection.Ascending ? 180 : 0;
        }

        private void TextBlock_MouseDown(object sender, MouseButtonEventArgs e)
        {
            TextBlock textBlock = (TextBlock)sender;

            string oldOrder = orderBy;

            switch (textBlock.Text)
            {
                case "N°":
                    orderBy = "Number";
                    break;
                case "Name":
                    orderBy = "Name";
                    break;
                case "Color":
                    orderBy = "Color.Name";
                    break;
                case "Rarity":
                    orderBy = "Rarity.Name";
                    break;
                case "Author":
                    orderBy = "Author.Name";
                    break;
                case "Exp.":
                    orderBy = "Expansion.Name";
                    break;
                case "Force":
                    orderBy = "Force";
                    break;
                case "Def.":
                    orderBy = "Defense";
                    break;
                case "Price":
                    orderBy = "Price";
                    break;
            }

            if (oldOrder != orderBy)
                sortDirection = ListSortDirection.Ascending;
            else if (sortDirection == ListSortDirection.Ascending)
                sortDirection = ListSortDirection.Descending;
            else
                sortDirection = ListSortDirection.Ascending;

            RefreshCardsGrid();

            MoveCursorToTextBlock(textBlock);
        }

        private void imagesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Card card = (Card)imagesList.SelectedItem;
            if (card != null && (card.CompletePath == null || card.Type == null || card.Text == null))
            {
                lock (imagesSearchingCards)
                {
                    imagesSearchingCards.Enqueue(card);
                    resetEvent.Set();
                }
            }
            imagesList.ScrollIntoView(imagesList.SelectedItem);

            if (card != null && card.Bitmap != null)
            {
                if (card.Bitmap.PixelWidth > card.Bitmap.PixelHeight)
                {
                    FullImage.Stretch = Stretch.Uniform;
                    FullImageTab.Height = 728 + 12;
                    FullImageTab.Width = 1043 + 18;
                }
                else
                {
                    FullImage.Stretch = Stretch.None;
                    FullImageTab.Height = 680 + 12;
                    FullImageTab.Width = 480 + 24;
                }
            }

            FullImage.DataContext = card;
        }

        private void cardsGrid_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void cardsGrid_DragOver(object sender, DragEventArgs e)
        {
            Point position = e.GetPosition(cardsGrid);

            HitTestResult result = VisualTreeHelper.HitTest(cardsGrid, position);

            if (result.VisualHit != null)
            {
                FrameworkElement element = (FrameworkElement)result.VisualHit;
                cardsGrid.SelectedItem = element.DataContext;
            }
        }

        private void cardsGrid_Drop(object sender, DragEventArgs e)
        {
            ChangeImage((Card)cardsGrid.SelectedItem, e);
        }

        private void blockTreeview_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void blockTreeview_DragOver(object sender, DragEventArgs e)
        {
            Point position = e.GetPosition(blockTreeview);

            HitTestResult result = VisualTreeHelper.HitTest(blockTreeview, position);

            if (result.VisualHit != null)
            {
                FrameworkElement element = (FrameworkElement)result.VisualHit;

                Tools.SetSelected(blockTreeview, element.DataContext);
            }
        }

        private void blockTreeview_Drop(object sender, DragEventArgs e)
        {
            try
            {
                string[] datas = (string[])e.Data.GetData("FileName");
                Block block = blockTreeview.SelectedItem as Block;

                if (block == null)
                {
                    Expansion expansion = blockTreeview.SelectedItem as Expansion;

                    if (Keyboard.Modifiers == ModifierKeys.Control)
                        expansion.Logo = File.ReadAllBytes(datas[0]);
                    else
                        expansion.Image = File.ReadAllBytes(datas[0]);
                }
                else
                {
                    block.Image = File.ReadAllBytes(datas[0]);
                }
            }
            catch (Exception)
            {
                MessageWindow.Show("Unable to update image.", "Urza Gatherer", this);
            }
        }

        private void nameFilters_TextChanged(object sender, TextChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void cardsGrid_KeyDown(object sender, KeyEventArgs e)
        {
            if (cardsGrid.SelectedItem == null)
                return;

            if (e.Key == Key.Enter)
            {
                Card card = (Card)cardsGrid.SelectedItem;
                card.Check = !card.Check;
                DataRoot.Entities.SaveChanges();
                cardsGrid.SelectedIndex++;
                UpdateCurrentCompleteLevel(card);

                ListViewItem itm = (ListViewItem)cardsGrid.ItemContainerGenerator.ContainerFromItem(cardsGrid.SelectedItem);
                itm.Focus();
                SynchroniseTreeview();
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            DoMainBinding();

            loadedDone = true;
        }

        private void DoMainBinding()
        {
            IOrderedQueryable<Block> blocks = DataRoot.Entities.Blocks.OrderByDescending(b => b.Index);
            blockTreeview.DataContext = blocks;

            colorFilters.ItemsSource = DataRoot.Entities.Colors.OrderBy(a => a.Order);
            colorFilters.SelectedIndex = 0;

            ColorsList.ItemsSource = DataRoot.Entities.Colors.OrderBy(a => a.Order);
            colorFilters.SelectedIndex = 0;

            rarityFilters.ItemsSource = DataRoot.Entities.Rarities.OrderBy(a => a.Order);
            rarityFilters.SelectedIndex = 0;

            RarityList.ItemsSource = DataRoot.Entities.Rarities.OrderBy(a => a.Order);
            RarityList.SelectedIndex = 0;

            authorFilters.DisplayMemberPath = "Name";
            authorFilters.ItemsSource = DataRoot.Entities.Authors.OrderBy(a => a.Name);
            authorFilters.SelectedIndex = 0;

            AuthorsList.DisplayMemberPath = "Name";
            AuthorsList.ItemsSource = DataRoot.Entities.Authors.OrderBy(a => a.Name);
        }

        private void flavorFilters_TextChanged(object sender, TextChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void imagesList_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void imagesList_DragOver(object sender, DragEventArgs e)
        {
            Point position = e.GetPosition(imagesList);

            HitTestResult result = VisualTreeHelper.HitTest(imagesList, position);

            if (result.VisualHit != null)
            {
                FrameworkElement element = (FrameworkElement)result.VisualHit;
                imagesList.SelectedItem = element.DataContext;
            }
        }

        void ChangeImage(Card card, DragEventArgs e)
        {
            try
            {
                string[] datas = (string[])e.Data.GetData("FileName");

                card.Image = File.ReadAllBytes(datas[0]);

            }
            catch (Exception)
            {
                MessageWindow.Show("Unable to update image.", "Urza Gatherer", this);
            }
        }

        private void imagesList_Drop(object sender, DragEventArgs e)
        {
            ChangeImage((Card)imagesList.SelectedItem, e);
        }

        private void typeFilters_TextChanged(object sender, TextChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void ResetClick(object sender, RoutedEventArgs e)
        {
            checkFilters.SelectedIndex = 0;
            colorFilters.SelectedIndex = 0;
            rarityFilters.SelectedIndex = 0;
            authorFilters.SelectedIndex = 0;
            nameFilters.Text = "";
            flavorFilters.Text = "";
            typeFilters.Text = "";
            fullTextFilters.Text = "";
            forceFilters.Text = "";
            defenseFilters.Text = "";
            textFilters.Text = "";
            priceFilters.Text = "";
        }

        private void AddExpansion_Click(object sender, RoutedEventArgs e)
        {
            Block block = blockTreeview.SelectedItem as Block;

            ExpansionImporter importer = new ExpansionImporter { Entities = DataRoot.Entities, ParentBlock = block };
            if (importer.ShowDialog() == true)
            {
                blockTreeview.ItemsSource = DataRoot.Entities.Blocks.OrderByDescending(b => b.Index);
            }
        }

        private void AddBlock_Click(object sender, RoutedEventArgs e)
        {
            QueryWindow query = new QueryWindow
                                    {
                                        Owner = this,
                                        Question = "Block Name:",
                                        Text = "",
                                        Title = "Add a new block"
                                    };

            if (query.ShowDialog() == true)
            {
                Block block = new Block
                                  {
                                      Name = query.Text,
                                      Index = (from b in DataRoot.Entities.Blocks select b.Index).Max() + 1
                                  };

                DataRoot.Entities.AddToBlocks(block);
                DataRoot.Entities.SaveChanges();

                blockTreeview.ItemsSource = DataRoot.Entities.Blocks.OrderByDescending(b => b.Index);
            }
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            int retry = 0;

            do
            {
                try
                {
                    DataRoot.Entities.SaveChanges();
                    return;
                }
                catch (Exception)
                {

                    retry++;
                }
            }
            while (retry < 5);

        }

        private void AffectHDPictures_Click(object sender, RoutedEventArgs e)
        {
            ImportPicturesWindow wnd = new ImportPicturesWindow
                                           {
                                               Expansion = blockTreeview.SelectedItem as Expansion
                                           };

            wnd.ShowDialog();
        }

        private void RemoveBlock_Click(object sender, RoutedEventArgs e)
        {
            if (QuestionWindow.Show("Are you sure you want to delete this block?", "Confirmation", this) == true)
            {
                Block block = blockTreeview.SelectedItem as Block;
                DataRoot.Entities.DeleteObject(block);

                DataRoot.Entities.SaveChanges();
            }
        }

        private void RemoveExpansion_Click(object sender, RoutedEventArgs e)
        {
            if (QuestionWindow.Show("Are you sure you want to delete this expansion?", "Confirmation", this) == true)
            {
                Expansion expansion = blockTreeview.SelectedItem as Expansion;
                DataRoot.Entities.DeleteObject(expansion);

                DataRoot.Entities.SaveChanges();

                blockTreeview.DataContext = DataRoot.Entities.Blocks.OrderByDescending(b => b.Index);
            }
        }

        private void BackupButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                VistaSaveFileDialog saveDialog = new VistaSaveFileDialog
                                                     {
                                                         Filter = @"Backup files|*.bak",
                                                         Title = @"Please select a filename",
                                                         AddExtension = true,
                                                         DefaultExt = ".bak"
                                                     };
                if (saveDialog.ShowDialog(this) == true)
                {
                    Cursor = Cursors.Wait;
                    Tools.DoEvents();

                    UrzaIncubator.Tools.Backup(saveDialog.FileName);

                    Cursor = Cursors.Arrow;
                    MessageWindow.Show("Backup successfull.", "UrzaGatherer", this);
                }
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Arrow;
                MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void RestoreButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                VistaOpenFileDialog openDialog = new VistaOpenFileDialog
                                                     {
                                                         Filter = @"All files|*.*",
                                                         Title = @"Please select a filename"
                                                     };
                if (openDialog.ShowDialog(this) == true)
                {
                    ProcessStartInfo info = new ProcessStartInfo(Path.Combine(System.Windows.Forms.Application.StartupPath, "DatabaseRestore.exe"), openDialog.FileName);
                    Process.Start(info);
                    Application.Current.Shutdown();
                }
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Arrow;
                MessageWindow.Show(ex.Message, "Error", this);
            }
        }

        private void MainHeader_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ToolBarSlideControl.SelectedIndex = MainHeader.SelectedIndex;
        }

        private void Grid_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 2)
            {
                CardEditor.DataContext = cardsGrid.SelectedItem;
                (FindResource("FlowDownStoryboard") as Storyboard).Begin();
                RootGrid.IsHitTestVisible = false;
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            CloseCardEditor(true);
        }

        void CloseCardEditor(bool save)
        {
            (FindResource("FlowUpStoryboard") as Storyboard).Begin();
            CardEditor.DataContext = null;
            RootGrid.IsHitTestVisible = true;
            if (save)
            {
                Card card = cardsGrid.SelectedItem as Card;

                card.ForceRefresh();

                if (itemAdded)
                    RefreshCardsGrid();
            }

            itemAdded = false;
        }

        private void Image_DragEnter(object sender, DragEventArgs e)
        {
            e.Effects = e.Data.GetDataPresent("FileName") ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void Image_Drop(object sender, DragEventArgs e)
        {
            ChangeImage((Card)imagesList.SelectedItem, e);
        }

        private void cardImage_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 2)
            {
                CardEditor.DataContext = cardsGrid.SelectedItem;
                (FindResource("FlowDownStoryboard") as Storyboard).Begin();
                RootGrid.IsHitTestVisible = false;
            }
        }

        private void RightHeader_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ImagesSlideControl.SelectedIndex = RightHeader.SelectedIndex;
        }

        private void forceFilters_TextChanged(object sender, TextChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void GridSplitter_DragDelta(object sender, System.Windows.Controls.Primitives.DragDeltaEventArgs e)
        {
            GridSplitter splitter = (GridSplitter)sender;
            int column = (int)splitter.GetValue(Grid.ColumnProperty);

            MoveCursorToColumn(column);
        }

        private void UpExpansion_Click(object sender, RoutedEventArgs e)
        {
            Expansion exp = (Expansion)blockTreeview.SelectedItem;

            exp.OrderInBlock--;

            exp.Block.RefreshSortedExpansions();

            Tools.SetSelected(blockTreeview, exp);
        }

        private void DownExpansion_Click(object sender, RoutedEventArgs e)
        {
            Expansion exp = (Expansion)blockTreeview.SelectedItem;

            exp.OrderInBlock++;

            exp.Block.RefreshSortedExpansions();

            Tools.SetSelected(blockTreeview, exp);
        }

        private void AddItem_Click(object sender, RoutedEventArgs e)
        {
            Expansion expansion = blockTreeview.SelectedItem as Expansion;
            Card card = new Card();
            card.Name = "New card";
            card.ID = (from c in DataRoot.Entities.Cards select c.ID).Max() + 1;
            card.Color = DataRoot.Entities.Colors.First();
            card.Author = DataRoot.Entities.Authors.First();
            card.Rarity = DataRoot.Entities.Rarities.First();
            card.Force = 0;
            card.Defense = 0;
            card.Expansion = expansion;
            expansion.Cards.Add(card);
            expansion.Complete = false;
            expansion.Block.Complete = false;
            DataRoot.Entities.SaveChanges();

            SynchroniseTreeview();

            cardsGrid.SelectedItem = card;

            CardEditor.DataContext = card;
            (FindResource("FlowDownStoryboard") as Storyboard).Begin();
            RootGrid.IsHitTestVisible = false;
            itemAdded = true;
        }

        private void CardEditor_Drop(object sender, DragEventArgs e)
        {
            ChangeImage((Card)CardEditor.DataContext, e);
        }

        private void textFilters_TextChanged(object sender, TextChangedEventArgs e)
        {
            RefreshCardsGrid();
        }

        private void Rescan_Click(object sender, RoutedEventArgs e)
        {
            if (QuestionWindow.Show("Are you sure you want to rescan all cards from gatherer.wizards.com?\nThe process can take several hours.", "Confirmation", this) == true)
            {
                Cursor = Cursors.Wait;
                Tools.DoEvents();
                Task task = Task.Factory.StartNew(() =>
                                          {
                                              try
                                              {
                                                  Card[] cards = DataRoot.Entities.Cards.ToArray();

                                                  int index = 1;
                                                  foreach (Card card in cards)
                                                  {
                                                      Debug.WriteLine(index++ + " : " + card.Name);
                                                      Tools.GetCardInfos(card);
                                                      DataRoot.Entities.SaveChanges();
                                                  }
                                                  MessageWindow.Show("Operation completed.", "UrzaGatherer", this);
                                              }
                                              catch (Exception ex)
                                              {
                                                  MessageWindow.Show(ex.Message, "Error during scan process", this);
                                              }
                                          });
                task.Wait();
                Cursor = Cursors.Arrow;
            }
        }

        private void DeleteItem_Click(object sender, RoutedEventArgs e)
        {
            if (cardsGrid.SelectedItem == null)
                return;
            if (QuestionWindow.Show("Are you sure you want to delete selected card?", "Confirmation", this) == true)
            {
                Card card = cardsGrid.SelectedItem as Card;
                DataRoot.Entities.DeleteObject(card);

                DataRoot.Entities.SaveChanges();
                cardsGrid.SelectedIndex = 0;
                SynchroniseTreeview();
            }
        }

        private void CardEditor_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Escape)
                CloseCardEditor(false);
            else if (e.Key == Key.Enter)
            {
                EditorCloseButton.Focus();
                CloseCardEditor(true);
            }
        }
    }
}
