﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using UrzaIncubator;
using mshtml;

namespace UrzaGatherer
{
    internal static class PriceManagerFallBack
    {
        private const double DeviceConvert = 0.7429;

        public enum CardNameMode
        {
            Normal,
            Suffix,
            None
        }

        private static string RemoveDiacritics(string stIn)
        {
            string stFormD = stIn.Normalize(NormalizationForm.FormD);
            var sb = new StringBuilder();

            for (var ich = 0; ich < stFormD.Length; ich++)
            {
                UnicodeCategory uc = CharUnicodeInfo.GetUnicodeCategory(stFormD[ich]);
                if (uc != UnicodeCategory.NonSpacingMark)
                {
                    sb.Append(stFormD[ich]);
                }
            }

            return (sb.ToString().Normalize(NormalizationForm.FormC)).Replace("!", "").Replace("&", "");
        }

        private static string GetCardUrl(Card card)
        {
            return card.Name.Replace("// ", "").Replace(" (v. 1)", "").Replace(" (v. 2)", " [version 2]").Replace(" (v. 3)", " [version 3]").Replace(" (v. 4)", " [version 4]").Replace("  [version 5]", "").Replace("  [version 6]", "").Replace(" ", "-");
        }

        private static string GetCardUrlWithSuffix(Card card, bool addParenthesis)
        {
            string result = card.Name.Replace("// ", "");
            for (int index = 0; index < 10; index++)
            {
                result = result.Replace(" (v. " + index + ")", "");
            }

            if (addParenthesis)
                return result.Replace(" ", "-") + "-(" + card.Number + ")";

            return result.Replace(" ", "-") + "-" + card.Number;
        }

        private static string GetCardUrlNone(Card card)
        {
            string result = card.Name.Replace("// ", "");
            for (int index = 0; index < 10; index++)
            {
                result = result.Replace(" (v. " + index + ")", "");
            }
            return result.Replace(" ", "-");
        }

        private static string GetExpansionUrl(Expansion expansion, Card card)
        {
            var name = expansion.Name.Replace(": City of Guilds", "");

            if (name.Contains("Alpha"))
                return "Alpha-edition";

            if (name.Contains("Beta"))
                return "Beta-edition";

            if (name.Contains("Classic"))
                return "classic-sixth-edition";

            if (name.Contains("Seventh"))
                return "7th-edition";

            if (name.Contains("Eight"))
                return "8th-edition";

            if (name.Contains("Ninth"))
                return "9th-edition";

            if (name.Contains("Tenth"))
                return "10th-edition";

            if (name.Contains("Magic 2010"))
                return "magic-2010-(m10)";

            if (name.Contains("Magic 2011"))
                return "magic-2011-(m11)";

            if (name.Contains("Magic 2012"))
                return "magic-2012-(m12)";

            if (name.Contains("Magic 2013"))
                return "magic-2013-(m13)";

            if (name.Contains("Timeshifted"))
                return "Timeshifted";

            if (name.Contains("Planechase 2012"))
                return "Planechase-2012";

            if (name.Contains("Duel Decks: Knights vs. Dragons"))
                return "duel-decks-knights-vs-dragons";

            if (card.Name == "Mirrored Depths" && name.Contains("Planechase"))
                return "planeshase";

            return name.Replace(":", "").Replace(" ", "-");

        }

        public static bool GetPriceLine(Expansion expansion, Card card, CardNameMode mode, bool removeDiacritics, bool addParenthesis = false)
        {
            string cardName = "";

            switch (mode)
            {
                case CardNameMode.Normal:
                    cardName = GetCardUrl(card);
                    break;
                case CardNameMode.Suffix:
                    cardName = GetCardUrlWithSuffix(card, addParenthesis);
                    break;
                case CardNameMode.None:
                    cardName = GetCardUrlNone(card);
                    break;
            }

            if (removeDiacritics)
                cardName = RemoveDiacritics(cardName);

            cardName = cardName.Replace(",", "").Replace("\"", "").Replace("/", "-");

            // Exceptions
            if (cardName.StartsWith("Our-Market-Research"))
                cardName = "our-market-research";
            else if (card.Name.StartsWith("Pang Tong"))
                cardName = "pang-tong-“young-phoenix”";
            else if (card.Name.StartsWith("Sun Ce,"))
                cardName = "sun-ce-young-conqueror";
            else if (card.Name.StartsWith("Kongming,"))
                cardName = "kongming-“sleeping-dragon”";
            else if (cardName.ToLower().StartsWith("the-ultimate-nightmare-of-wizards"))
                cardName = "the-ultimate-nightmare-of-wizards-of-the-coast-cu";
            else if (card.Name == "B.F.M. (Big Furry Monster)")
            {
                if (card.Number == 28)
                    cardName = "b.f.m.-(big-furry-monster-left)";
                else
                    cardName = "b.f.m.-(big-furry-monster-right)";
            }

            // URL
            var url = string.Format(@"http://store.tcgplayer.com/magic/{0}/{1}", GetExpansionUrl(expansion, card), cardName);

            string data = Tools.GetData(url);

            if (string.IsNullOrEmpty(data))
            {
                return false;
            }

            const string lookup = "<td class=\"avg\">";
            const string lookupBis = "<td class=\"price\">";
            var pos = data.IndexOf(lookup, StringComparison.Ordinal);

            if (pos == -1)
            {
                if (mode == CardNameMode.Suffix && !addParenthesis)
                    return GetPriceLine(expansion, card, CardNameMode.Suffix, removeDiacritics, true);
                return false;
            }

            var dataStart = pos + lookup.Length;
            var dataEnd = data.IndexOf("</td>", dataStart, StringComparison.Ordinal);
            var result = data.Substring(dataStart, dataEnd - dataStart);
            var cardPrice = double.Parse(result.Replace("$", "").Trim(), CultureInfo.InvariantCulture) * DeviceConvert;

            if (cardPrice == 0)
            {
                pos = data.IndexOf(lookupBis, StringComparison.Ordinal);

                if (pos != -1)
                {
                    dataStart = pos + lookupBis.Length;
                    dataEnd = data.IndexOf("<", dataStart, StringComparison.Ordinal);
                    result = data.Substring(dataStart, dataEnd - dataStart);
                    cardPrice = double.Parse(result.Replace("$", "").Trim(), CultureInfo.InvariantCulture) * DeviceConvert;
                }
            }

            lock (expansion)
            {
                card.Price = cardPrice;
                card.CardMarketLink = url;
            }
            return true;
        }
    }
}