﻿using System;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UrzaIncubator;
using System.Threading;
using System.Diagnostics;
using System.IO;
using System.Globalization;

namespace UrzaGatherer
{
    internal static class PriceManager
    {
        const int PriceIndex = 22;
        const double DeviceConvert = 0.7642;

        static string RemoveDiacritics(string stIn)
        {
            string stFormD = stIn.Normalize(NormalizationForm.FormD);
            var sb = new StringBuilder();

            for (var ich = 0; ich < stFormD.Length; ich++)
            {
                UnicodeCategory uc = CharUnicodeInfo.GetUnicodeCategory(stFormD[ich]);
                if (uc != UnicodeCategory.NonSpacingMark)
                {
                    sb.Append(stFormD[ich]);
                }
            }

            return (sb.ToString().Normalize(NormalizationForm.FormC)).Replace("!", "").Replace("&", "");
        }

        static string PrepareExpansionName(string expansionName, Card card)
        {
            var name = expansionName.Replace(": City of Guilds", "");

            if (name.Contains("Alpha"))
                return "Alpha edition";

            if (name.Contains("Beta"))
                return "Beta edition";

            if (name.Contains("Seventh"))
                return "7th edition";

            if (name.Contains("Eight"))
                return "8th edition";

            if (name.Contains("Ninth"))
                return "9th edition";

            if (name.Contains("Tenth"))
                return "10th edition";

            if (name.Contains("Magic 2010"))
                return "magic 2010 (m10)";

            if (name.Contains("Magic 2011"))
                return "magic 2011 (m11)";

            if (name.Contains("Magic 2012"))
                return "magic 2012 (m12)";

            if (name.Contains("Magic 2013"))
                return "magic 2013 (m13)";

            if (name.Contains("Timeshifted"))
                return "Timeshifted";

            if (name.Contains("Planechase 2012"))
                return "Planechase 2012";

            if (name.Contains("Duel Decks: Knights vs. Dragons"))
                return "Duel Decks: Knights vs Dragons";

            if (card.Name == "Mirrored Depths" && name.Contains("Planechase"))
                return "planeshase";

            return name;

            return name.Replace(":", "").Replace(" ", "-");

        }

        static bool GetPriceLine(Expansion expansion, Card card, bool replace, bool removeDiacritics)
        {
            string url = "http://partner.tcgplayer.com/x2/phl.asmx/p?pk=TCGTEST&s=" + PrepareExpansionName(expansion.Name, card) + "&p=" + PrepareCardName(card.Name, replace, removeDiacritics);
            string data = Tools.GetData(url);

            var text = GetValue(data, "avgprice");

            if (text == null)
                return false;

            var cardPrice = double.Parse(text.Replace("$", "").Trim(), CultureInfo.InvariantCulture) * DeviceConvert;

            Debug.WriteLine("->{0}: {1}$", card.Name, cardPrice);
            url = GetValue(data, "link");

            lock (expansion)
            {
                card.Price = cardPrice;
                card.CardMarketLink = url;
            }

            return true;
        }

        static string GetValue(string data, string tag)
        {
            string lookup = "<" + tag + ">";
            var pos = data.IndexOf(lookup, StringComparison.Ordinal);
            if (pos == -1)
                return null;
            var dataStart = pos + lookup.Length;
            var dataEnd = data.IndexOf("</" + tag + ">", dataStart, StringComparison.Ordinal);
            return data.Substring(dataStart, dataEnd - dataStart);
        }

        static string PrepareCardName(string cardName, bool replace, bool removeDiacritics)
        {
            cardName = cardName.Replace("’", "'").Replace("Æ", "AE");

            if (removeDiacritics)
                cardName = RemoveDiacritics(cardName);

            for (int index = 0; index < 10; index++)
            {
                if (replace)
                {
                    cardName = cardName.Replace(string.Format(" (v. {0})", index), string.Format(" [version {0}]", index));
                    cardName = cardName.Replace(string.Format(" (ver. {0})", index), string.Format(" [version {0}]", index));
                    cardName = cardName.Replace(string.Format(" (v.{0})", index), string.Format(" [version {0}]", index));
                }
                else
                {
                    cardName = cardName.Replace(string.Format(" (v. {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (ver. {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (v.{0})", index), "");
                }
            }

            return cardName;
        }

        public static void GetExpansionPrice(Expansion expansion, Action<int> progressCallback = null)
        {
            Debug.WriteLine("===== Checking " + expansion.Name);
            if (!expansion.PriceUpdateIndex.HasValue)
                expansion.PriceUpdateIndex = 0;

            if (expansion.PriceUpdateIndex >= PriceIndex)
                return;

            int total = 0;

            var cards = expansion.Cards.ToArray();
            var controlSemaphore = new Semaphore(8, 8);
            Parallel.ForEach(cards, card =>
                                    {
                                        controlSemaphore.WaitOne();
                                        Debug.WriteLine("=== Processing " + card.Name);

                                        if (!GetPriceLine(expansion, card, true, true) && !GetPriceLine(expansion, card, false, true) && !GetPriceLine(expansion, card, true, false) && !GetPriceLine(expansion, card, false, false)
                                                && !PriceManagerFallBack.GetPriceLine(expansion, card, PriceManagerFallBack.CardNameMode.Normal, true)
                                                && !PriceManagerFallBack.GetPriceLine(expansion, card, PriceManagerFallBack.CardNameMode.Suffix, true)
                                                && !PriceManagerFallBack.GetPriceLine(expansion, card, PriceManagerFallBack.CardNameMode.None, true)
                                                && !PriceManagerFallBack.GetPriceLine(expansion, card, PriceManagerFallBack.CardNameMode.Normal, false)
                                                && !PriceManagerFallBack.GetPriceLine(expansion, card, PriceManagerFallBack.CardNameMode.Suffix, false)
                                                && !PriceManagerFallBack.GetPriceLine(expansion, card, PriceManagerFallBack.CardNameMode.None, false))
                                        {
                                            if (card.Tag != "b")
                                            {
                                                lock (expansion)
                                                {
                                                    var path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "pricesIncomplete.txt");
                                                    var textToAppend = string.Format("{0} not found for {1}\r\n", card.Name, expansion.Name);
                                                    File.AppendAllText(path, textToAppend);
                                                    Debug.WriteLine(textToAppend);
                                                    card.CardMarketLink = "";
                                                }

                                            }
                                            else
                                            {                                                
                                                total++;
                                            }
                                        }
                                        else
                                        {
                                            total++;
                                        }

                                        card.CardMarketLink = card.CardMarketLink.Replace("TCGTEST", "URZGTHR");

                                        if (progressCallback != null)
                                        {
                                            progressCallback(1);
                                        }
                                        controlSemaphore.Release();
                                    });

            Debug.WriteLine(string.Format("===================================== Processed {0} card on {1} total ", total, expansion.Cards.Count));

            if (total >= expansion.Cards.Count)
            {
                expansion.PriceUpdateIndex = PriceIndex;
                Debug.WriteLine(string.Format("===================================== Scan for {0} successfull", expansion.Name));
            }
            else
            {
                var path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "pricesIncomplete.txt");
                var text = string.Format("================>Incomplete scan for {0} : {1}/{2}\r\n", expansion.Name, total, expansion.Cards.Count);
                File.AppendAllText(path, text);
                Debug.WriteLine(text);
            }

            if (progressCallback != null)
            {
                progressCallback(expansion.Cards.Count() - total);
            }

            lock (DataRoot.Entities)
            {
                DataRoot.Entities.SaveChanges();
            }
        }
    }
}