﻿using System;
using System.Collections.Generic;
using System.Data.SqlServerCe;
using System.IO;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using mshtml;
using UrzaIncubator;

namespace UrzaGatherer
{
    internal static class Tools
    {
        static readonly Dictionary<string, string> expansionsSmallCodes = new Dictionary<string, string>();

        public static void DoEvents()
        {
            Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, new ThreadStart(delegate { }));
        }

        public static void ConnectEntities()
        {
            try
            {
                DataRoot.Entities = UrzaGathererEntities.Create();
            }
            catch (Exception)
            {
                if (DataRoot.Entities != null)
                    DataRoot.Entities.Dispose();
                throw;
            }
        }

        public static bool SetSelected(ItemsControl parent, object child)
        {
            if (parent == null || child == null)
            {
                return false;
            }

            TreeViewItem childNode = parent.ItemContainerGenerator.ContainerFromItem(child) as TreeViewItem;

            if (childNode != null)
            {
                childNode.Focus();
                return childNode.IsSelected = true;
            }

            if (parent.Items.Count > 0)
            {
                foreach (object childItem in parent.Items)
                {
                    ItemsControl childControl = parent.ItemContainerGenerator.ContainerFromItem(childItem) as ItemsControl;
                    if (SetSelected(childControl, child))
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        public static bool GetCardImage(Card card)
        {
            string cardName = card.Name.ToLower();

            cardName = GetCardCorrectName(cardName, true);
            cardName = cardName.Replace(" ", "_");
            cardName = cardName.Replace("'", "");
            cardName = cardName.Replace("’", "");
            cardName = cardName.Replace("-", "_");
            cardName = cardName.Replace(",", "");
            cardName = cardName.Replace(":", "");

            string expansionName = card.Expansion.Name;
            expansionName = expansionName.Replace(" ", "_");
            expansionName = expansionName.Replace("'", "");
            expansionName = expansionName.Replace(":", "");

            WebRequest request = WebRequest.Create(string.Format("http://www.magiccardmarket.eu/img/cards/{0}/{1}.jpg", expansionName, cardName));

            try
            {
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                if (response.ContentType == "text/html")
                    return false;
                Stream stream = response.GetResponseStream();
                MemoryStream memStream = new MemoryStream();

                byte[] buffer = new byte[4096];
                int count;
                do
                {
                    count = stream.Read(buffer, 0, buffer.Length);
                    if (count > 0)
                        memStream.Write(buffer, 0, count);
                }
                while (count > 0);

                card.Image = memStream.GetBuffer();

                memStream.Dispose();
                stream.Dispose();
                return true;
            }
            catch
            {
                return false;
            }
        }

        internal static string GetCardCorrectName(string cardName, bool keepIndex)
        {
            cardName = cardName.Replace("û", "u");
            cardName = cardName.Replace("à", "a");
            cardName = cardName.Replace("á", "a");
            cardName = cardName.Replace("ö", "o");
            cardName = cardName.Replace("æ", "ae");

            for (int index = 1; index < 10; index++)
            {
                if (keepIndex)
                {
                    cardName = cardName.Replace(string.Format(" (v. {0})", index), index.ToString());
                    cardName = cardName.Replace(string.Format(" (ver. {0})", index), index.ToString());
                    cardName = cardName.Replace(string.Format(" (version {0})", index), index.ToString());
                    cardName = cardName.Replace(string.Format(" (v.{0})", index), index.ToString());
                }
                else
                {
                    cardName = cardName.Replace(string.Format(" (v. {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (ver. {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (version {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (v.{0})", index), "");
                }
            }
            return cardName;
        }

        public static bool GetCardImage2(Card card, string language)
        {
            string expansionName;
            WebRequest rootRequest = WebRequest.Create("http://magiccards.info/sitemap.html");

            if (!expansionsSmallCodes.TryGetValue(card.Expansion.Name, out expansionName))
            {
                string dataMap;
                using (StreamReader reader = new StreamReader(rootRequest.GetResponse().GetResponseStream()))
                {
                    dataMap = reader.ReadToEnd();
                }

                Regex regex = new Regex(string.Format(@"({0}</a> <small style=""color: \#aaa;"">)(?<small>.+)(</small>)",
                                            card.Expansion.Name));

                Match match = regex.Match(dataMap);

                expansionName = match.Groups[3].Value;

                expansionsSmallCodes.Add(card.Expansion.Name, expansionName);
            }

            WebRequest request = WebRequest.Create(string.Format("http://magiccards.info/scans/{2}/{0}/{1}.jpg", expansionName, card.Number, language));

            try
            {
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                if (response.ContentType == "text/html")
                    return false;
                Stream stream = response.GetResponseStream();
                MemoryStream memStream = new MemoryStream();

                byte[] buffer = new byte[4096];
                int count;
                do
                {
                    count = stream.Read(buffer, 0, buffer.Length);
                    if (count > 0)
                        memStream.Write(buffer, 0, count);
                }
                while (count > 0);

                card.Image = memStream.GetBuffer();

                memStream.Dispose();
                stream.Dispose();
                return true;
            }
            catch
            {
                return false;
            }
        }

        internal static IHTMLElement GetElementByInnerText(this IHTMLElement root, string innerText)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (el.innerText == innerText)
                {
                    return el;
                }
            }

            return null;
        }

        internal static List<IHTMLElement> GetElementsByClass(this IHTMLElement root, string id)
        {
            List<IHTMLElement> result = new List<IHTMLElement>();
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (!string.IsNullOrEmpty(el.className) && el.className.ToLower() == id.ToLower())
                {
                    result.Add(el);
                }
            }

            return result;
        }

        internal static IHTMLElement GetElementByClass(this IHTMLElement root, string id)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (!string.IsNullOrEmpty(el.className) && el.className.ToLower() == id.ToLower())
                {
                    return el;
                }
            }

            return null;
        }

        internal static string GetElementTextByClass(this IHTMLElement root, string id)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (!string.IsNullOrEmpty(el.className) && el.className.ToLower() == id.ToLower())
                {
                    return el.innerText;
                }
            }

            return "";
        }

        internal static string GetElementTextByClassIncomplete(this IHTMLElement root, string id)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (!string.IsNullOrEmpty(el.className) && el.className.ToLower().StartsWith(id.ToLower()))
                {
                    return el.innerText;
                }
            }

            return "";
        }

        internal static string GetAnchorText(this IHTMLElement root)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (el is IHTMLAnchorElement)
                {
                    return el.innerText;
                }
            }

            return "";
        }

        internal static string GetAnchorHref(this IHTMLElement root)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                IHTMLAnchorElement anchor = el as IHTMLAnchorElement;
                if (anchor != null)
                {
                    return anchor.href;
                }
            }

            return "";
        }

        internal static IHTMLElement GetElementByID(this IHTMLDocument2 document, string id)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)document.body.all)
            {
                var attribute = el.getAttribute("id");
                if (attribute != null && attribute.ToString().ToLower() == id.ToLower())
                {
                    return el;
                }
            }

            return null;
        }

        internal static IHTMLElement GetAElementByInnerText(this IHTMLDocument2 document, string innerText, bool startOnly = false)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)document.body.all)
            {
                if (el.tagName == "A" && !string.IsNullOrEmpty(el.innerText))
                {
                    if (startOnly)
                    {
                        if (el.innerText.ToLower().StartsWith(innerText.ToLower()))
                            return el;
                    }
                    else if (string.Compare(el.innerText, innerText, true) == 0)
                        return el;
                }
            }

            return null;
        }

        internal static IHTMLElement GetImgElementByTitle(this IHTMLDocument2 document, string title, bool startOnly = false)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)document.body.all)
            {
                if (el.tagName == "IMG")
                {
                    var attribute = el.getAttribute("title");

                    if (attribute != null)
                    {
                        if (startOnly)
                        {
                            if (attribute.ToString().ToLower().StartsWith(title.ToLower()))
                                return el;
                        }
                        else if (string.Compare(attribute.ToString(), title, true) == 0)
                            return el;
                    }
                }
            }

            return null;
        }

        public static void GetCardInfos(Card card, bool requestAuthor = true)
        {
            if (card.Type != null && card.Power != null && card.Flavor != null && card.FrenchFlavor != null && card.FrenchName != null && card.Text != null)
                return;
            try
            {
                string[] splits = card.Name.Replace("'s", " ").Replace("'", " ").Replace("’s", " ").Replace("’", " ").Replace(@"""", "").Replace("!", "").Replace("&", " ").Split();

                string requestName = "";
                string linkName = "";

                foreach (string split in splits)
                {
                    if (split.StartsWith("(v.") || split.StartsWith("(ver."))
                        break;
                    if (!string.IsNullOrEmpty(split))
                    {
                        linkName += string.IsNullOrEmpty(linkName) ? split : " " + split;
                        requestName += "+[" + split + "]";
                    }
                }

                string requestExpansion = string.Format(@"[""{0}""]", card.Expansion.Name);

                string requestArtist = "";

                if (requestAuthor)
                    requestArtist = string.Format(@"&artist=[""{0}""]", card.Author.Name);

                string url = string.Format(@"http://gatherer.wizards.com/Pages/Search/Default.aspx?action=advanced&name={0}&set={1}{2}", requestName, requestExpansion, requestArtist);

                string data = GetData(url);
                IHTMLDocument2 document = (IHTMLDocument2)new HTMLDocument();
                document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                document.close();
                IHTMLElement element = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_typeRow");

                if (element == null)
                {
                    IHTMLElement linkElement = (document.GetAElementByInnerText(linkName) ??
                                                document.GetAElementByInnerText(card.Name)) ?? document.GetAElementByInnerText(card.Name, true);

                    if (linkElement != null)
                    {
                        data = GetData(linkElement.getAttribute("href").ToString().Replace("..", "http://gatherer.wizards.com/Pages").Replace("about:", ""));
                        if (data != null)
                        {
                            document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                            document.close();
                            element = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_typeRow");
                        }
                    }

                    if (element == null)
                    {
                        if (requestAuthor)
                            GetCardInfos(card, false);
                        return;
                    }
                }

                IHTMLElement expansionElement = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_currentSetSymbol");
                if (string.Compare(expansionElement.innerText.Trim(), card.Expansion.Name) != 0)
                {
                    IHTMLElement goodExpansion = document.GetImgElementByTitle(card.Expansion.Name, true);
                    data = GetData("http://gatherer.wizards.com/Pages/Card/" + goodExpansion.parentElement.getAttribute("href").ToString().Replace("about:", ""));
                    if (data != null)
                    {
                        document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                        document.close();
                        element = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_typeRow");
                    }
                }

                if (card.Type == null)
                {
                    card.Type = element.children[1].innerText.Trim().Replace("\r", "").Replace("\n", "");
                    if (card.Color.Name == "Artifact" && card.Type.ToLower().Contains("land") && !card.Type.ToLower().Contains("artifact"))
                        card.Color = (from c in DataRoot.Entities.Colors where c.Name == "Land" select c).First();
                }

                IHTMLElement textElement = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_textRow");
                if (textElement != null)
                {
                    if (card.Text == null)
                    {
                        card.Text = ConvertHTML2Text(textElement.children[1].children);
                    }
                }
                else
                {
                    card.Text = "";
                }

                if (card.Power == null)
                {
                    element = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_ptRow");
                    if (element != null)
                    {
                        card.Power = element.children[1].innerText.Trim().Replace("\r", "").Replace("\n", "");
                    }
                    else
                    {
                        card.Power = "";
                    }
                    card.ExtractForceAndDefense();
                }

                // Flavor
                if (card.Flavor == null)
                {
                    element = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_FlavorText");

                    if (element != null)
                    {
                        card.Flavor = ExtractFlavor(element);
                    }
                    else
                    {
                        card.Flavor = "";
                        card.FrenchFlavor = "";
                    }
                }

                // French flavor
                if (card.Expansion.FrenchVersion && (card.FrenchFlavor == null || card.FrenchName == null))
                {
                    bool found = false;
                    bool needFrenchFlavor = false;
                    bool needFrenchName = false;

                    if (card.FrenchFlavor == null)
                    {
                        needFrenchFlavor = true;
                        card.FrenchFlavor = "";
                    }

                    if (card.FrenchName == null)
                    {
                        needFrenchName = true;
                        card.FrenchName = "";
                    }

                    element = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContentAnchors_DetailsAnchors_LanguagesLink");
                    data = GetData("http://gatherer.wizards.com/Pages/Card/" + element.getAttribute("href").ToString().Replace("about:", ""));
                    if (data != null)
                    {
                        document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                        document.close();

                        for (int index = 0; index < 25; index++)
                        {
                            element = document.GetElementByID(string.Format("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_languageList_listRepeater_ctl0{0}_cardTitle", index));

                            if (element != null)
                            {
                                if (element.parentElement.parentElement.children[1].innerText.Trim() == "French")
                                {
                                    found = true;
                                    if (needFrenchName)
                                    {
                                        card.FrenchName = element.innerText.Trim();
                                    }
                                    if (needFrenchFlavor)
                                    {
                                        data = GetData("http://gatherer.wizards.com/Pages/Card/" + element.getAttribute("href").ToString().Replace("about:", ""));
                                        if (data != null)
                                        {
                                            document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                                            document.close();

                                            element = document.GetElementByID("ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_FlavorText");

                                            if (element != null)
                                            {
                                                card.FrenchFlavor = ExtractFlavor(element);
                                            }
                                        }
                                    }
                                    break;
                                }
                            }
                            else
                            {
                                break;
                            }
                        }
                        if (!found)
                            card.Expansion.FrenchVersion = false;
                    }

                }

            }
            catch
            {
            }
        }

        private static string ConvertHTML2Text(dynamic elements)
        {
            string result = "";
            foreach (IHTMLElement element in elements)
            {
                result += ConvertHTMLElement2Text(element) + "\r\n";
            }

            return result;
        }

        static string ConvertHTMLElement2Text(IHTMLElement root)
        {
            if (root.children.length == 0)
                return root.innerText;

            string result = root.innerHTML;
            foreach (IHTMLElement element in root.children)
            {
                if (element.tagName == "IMG")
                {
                    string replaceString = "";

                    for (int index = 0; index < 10; index++)
                    {
                        if (element.outerHTML.Contains("name=" + index))
                            replaceString = index.ToString();
                    }

                    if (element.outerHTML.Contains("name=X"))
                        replaceString = "X";

                    if (element.outerHTML.Contains("name=tap"))
                        replaceString = "T";

                    if (element.outerHTML.Contains("name=W"))
                        replaceString = "W";

                    if (element.outerHTML.Contains("name=R"))
                        replaceString = "R";

                    if (element.outerHTML.Contains("name=G"))
                        replaceString = "G";

                    if (element.outerHTML.Contains("name=U"))
                        replaceString = "U";

                    if (element.outerHTML.Contains("name=B"))
                        replaceString = "B";

                    result = result.Replace(element.outerHTML, replaceString);
                }
                else
                {
                    result = result.Replace(element.outerHTML, element.innerText);
                }

                result = result.Replace("<I>", "").Replace("<i>", "").Replace("</I>", "").Replace("</i>", "");
            }

            return result;
        }

        private static string ExtractFlavor(IHTMLElement element)
        {
            string flavor = "";
            foreach (IHTMLElement child in element.children)
            {
                if (child.innerText != null)
                {
                    string value = child.innerText.Trim().Replace("\r", "").Replace("\n", "");

                    if (!string.IsNullOrEmpty(value))
                        flavor += value + "\n";
                }
            }
            return flavor;
        }

        internal static string GetData(string url)
        {
            if (!url.StartsWith("http://", StringComparison.InvariantCultureIgnoreCase))
                return null;

            int retry = 0;
            do
            {
                try
                {
                    WebRequest rootRequest = WebRequest.Create(url);
                    string data;
                    using (StreamReader reader = new StreamReader(rootRequest.GetResponse().GetResponseStream()))
                    {
                        data = reader.ReadToEnd();
                    }
                    return data;
                }
                catch
                {
                    retry++;
                }
            }
            while (retry < 5);

            return null;
        }

        public static void CleanAuthors(UrzaGathererEntities urzaGathererEntities)
        {
            Author[] authors = urzaGathererEntities.Authors.ToArray();

            for (int index = 0; index < authors.Length; index++)
            {
                Author author = authors[index];

                if (author.EntityState == System.Data.EntityState.Deleted)
                    continue;
                var query = from a in urzaGathererEntities.Authors
                            where string.Compare(a.Name, author.Name, true) == 0 && a.ID != author.ID
                            select a;

                List<Author> duplicates = query.ToList();
                foreach (Author duplicate in duplicates)
                {
                    List<Card> cards = duplicate.Cards.ToList();
                    foreach (Card c in cards)
                    {
                        c.Author = author;
                    }
                    urzaGathererEntities.DeleteObject(duplicate);
                }

            }
            urzaGathererEntities.SaveChanges();
        }
    }
}
