﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;
using mshtml;
using UrzaGatherer.ViewModels;
using UrzaIncubator;

namespace UrzaGatherer
{
    public partial class ExpansionImporter
    {
        bool cancelRequested;
        readonly RootViewModel rootViewModel;

        public ExpansionImporter()
        {
            InitializeComponent();

            rootViewModel = (Application.Current as App).RootViewModel;
        }


        void ImportExpansion()
        {
            bool importPictures = ImportFromMagicCardsInfo.IsChecked == true;
            rootViewModel.Commit();
            LogText.Text = "Importation started.\n\n";

            Expansion expansion = null;
            string expansionName = ExpansionName.Text;

            if (string.IsNullOrEmpty(expansionName))
            {
                LogText.Text += "Invalid expansion name.";
                return;
            }

            Task.Factory.StartNew(() =>
             {
                 try
                 {
                     int mountains = 1;
                     int forests = 1;
                     int plains = 1;
                     int swamps = 1;
                     int islands = 1;
                     string url = string.Format("http://gatherer.wizards.com/Pages/Search/Default.aspx?output=checklist&set=[\"{0}\"]&special=true", expansionName);

                     string data = Tools.GetData(url);
                     IHTMLDocument2 document = (IHTMLDocument2)new HTMLDocument();
                     document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                     document.close();
                     List<IHTMLElement> elements = document.body.GetElementsByClass("cardItem");

                     if (elements.Count == 0)
                     {
                         Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                           {
                               LogText.Text += "Expansion not found.";
                           }));
                         return;
                     }

                     Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                        {
                            ImportationProgress.Maximum = elements.Count - 1;
                            ImportationProgress.Value = 0;

                            expansion = rootViewModel.BlockViewModel.CreateExpansion(expansionName);
                        }));

                     bool numberNotFound = false;
                     int localNumber = 1;
                     foreach (IHTMLElement element in elements)
                     {
                         if (cancelRequested)
                             return;

                         Card card = rootViewModel.ExpansionViewModel.CreateCard(expansion);

                         if (!numberNotFound)
                         {
                             try
                             {
                                 card.Number = int.Parse(element.GetElementTextByClass("number"));
                             }
                             catch
                             {
                                 numberNotFound = true;
                             }
                         }

                         if (numberNotFound)
                             card.Number = localNumber++;

                         var name = element.GetElementTextByClass("name");

                         card.Name = name;
                         card.Author = rootViewModel.ReferencesViewModel.GetAuthorFromName(element.GetElementTextByClass("artist"));
                         card.Color = rootViewModel.ReferencesViewModel.GetColorFromName(element.GetElementTextByClass("color"));
                         card.RarityID = rootViewModel.ReferencesViewModel.GetRarityFromName(element.GetElementTextByClass("rarity"));

                         if (importPictures)
                             Tools.GetCardImage2(card, "en");
                         Tools.GetCardInfos(card);

                         switch (name)
                         {
                             case "Island":
                                 card.Name = string.Format("{0} (v. {1})", name, islands++);
                                 break;
                             case "Mountain":
                                 card.Name = string.Format("{0} (v. {1})", name, mountains++);
                                 break;
                             case "Plains":
                                 card.Name = string.Format("{0} (v. {1})", name, plains++);
                                 break;
                             case "Swamp":
                                 card.Name = string.Format("{0} (v. {1})", name, swamps++);
                                 break;
                             case "Forest":
                                 card.Name = string.Format("{0} (v. {1})", name, forests++);
                                 break;
                             default:
                                 break;
                         }

                         Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                         {
                             ImportationProgress.Value++;
                             ImportationInfo.Text = string.Format("Importing {0} - {1:0.00}%",
                                                                  card.Name,
                                                                  ((ImportationProgress.Value + 1) * 100) /
                                                                  elements.Count);
                             LogText.Text += string.Format("[{3:000}] {0} - {1} - {2}\n",
                                                           card.Name, card.Color.Name,
                                                           card.Author.Name, card.Number);
                             LogText.ScrollToEnd();
                         }));
                     }

                     Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                        {
                            Tools.CleanAuthors(DataRoot.Entities);
                        }));

                     rootViewModel.Commit();
                     Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                     {
                         CancelButton.Visibility = Visibility.Collapsed;
                         AcceptButton.Visibility = Visibility.Visible;
                         LogText.Text += "\n\nImportation done.";
                         LogText.ScrollToEnd();
                         ImportationInfo.Text = "Importation done";
                     }));
                 }
                 catch (Exception ex)
                 {
                     if (cancelRequested)
                         return;
                     MessageWindow.Show("Error : " + ex.Message, "Urza Gatherer", this);
                     Cancel();
                 }
             });
        }

        private void Cancel()
        {
            cancelRequested = true;
            rootViewModel.Rollback();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Cancel();
            DialogResult = false;
        }

        private void Launch_Click(object sender, RoutedEventArgs e)
        {
            CancelButton.IsEnabled = true;
            Launch.IsEnabled = false;
            ExpansionName.IsEnabled = false;
            ImportFromMagicCardsInfo.IsEnabled = false;
            ImportExpansion();
        }

        private void AcceptButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }
    }
}
