﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.IO;
using Ookii.Dialogs.Wpf;
using UrzaGatherer.ViewModels;
using UrzaIncubator;

namespace UrzaGatherer
{
    public partial class ImportPicturesWindow
    {
        readonly ExpansionViewModel expansionViewModel;

        public ImportPicturesWindow()
        {
            InitializeComponent();

            expansionViewModel = (Application.Current as App).RootViewModel.ExpansionViewModel;

            expansionViewModel.PicturesImportationProgressChanged += expansionViewModel_PicturesImportationProgressChanged;
            expansionViewModel.PicturesImportationProgressCompleted += expansionViewModel_PicturesImportationProgressCompleted;
            expansionViewModel.PicturesImportationCardNotFound += expansionViewModel_PicturesImportationCardNotFound;
        }

        void expansionViewModel_PicturesImportationCardNotFound(Card card)
        {
            CardsList.Items.Add(card);
        }

        void expansionViewModel_PicturesImportationProgressCompleted(FileInfo[] files)
        {
            foreach (FileInfo file in files)
            {
                ImagesList.Items.Add(file);
            }
            CloseButton.IsEnabled = true;
        }

        void expansionViewModel_PicturesImportationProgressChanged(int progress)
        {
            Progress.Value = progress;
            TextReport.Text = string.Format("{0:0.##} %", (Progress.Value * 100.0) / Progress.Maximum);
        }

        private void BrowseButton_Click(object sender, RoutedEventArgs e)
        {
            VistaFolderBrowserDialog dialog = new VistaFolderBrowserDialog
                                                  {
                                                      Description = @"Please select a folder containing pictures for the expansion.", 
                                                      ShowNewFolderButton = false, 
                                                      UseDescriptionForTitle = true
                                                  };
            if (dialog.ShowDialog() == true)
            {
                TextPath.Text = dialog.SelectedPath;
                DoImport();
            }
        }

        private void DoImport()
        {
            Progress.Maximum = expansionViewModel.Current.Cards.Count;
            Progress.Value = 0;
            CloseButton.IsEnabled = false;
            CardsList.Items.Clear();
            ImagesList.Items.Clear();

            expansionViewModel.ImportPictures(TextPath.Text);
        }

        private void AffectButton_Click(object sender, RoutedEventArgs e)
        {
            if (CardsList.SelectedItem == null)
                return;
            if (ImagesList.SelectedItem == null)
                return;

            Card card = CardsList.SelectedItem as Card;
            string path = ImagesList.SelectedItem.ToString();

            card.Image = File.ReadAllBytes(Path.Combine(TextPath.Text, path));

            CardsList.Items.Remove(card);
            ImagesList.Items.RemoveAt(ImagesList.SelectedIndex);
        }

        private void ImagesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (ImagesList.SelectedItem == null)
            {
                ImagePreview.Source = null;
                return;
            }
            string path = ImagesList.SelectedItem.ToString();
            byte[] bytes = File.ReadAllBytes((Path.Combine(TextPath.Text, path)));

            ImageSourceConverter converter = new ImageSourceConverter();

            ImagePreview.Source = (ImageSource)converter.ConvertFrom(bytes);
        }

        private void Close_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }
    }
}
