﻿using System;
using System.IO;
using System.Windows.Media.Imaging;

namespace UrzaIncubator
{
    public partial class Card
    {
        BitmapImage bitmapImage;

        public BitmapFrame PictureFrame { get; set; }

        public BitmapImage Bitmap
        {
            get
            {
                if (CompletePath == null)
                    return null;

                if (bitmapImage == null)
                {
                    bitmapImage = new BitmapImage();
                    bitmapImage.BeginInit();
                    bitmapImage.CreateOptions = BitmapCreateOptions.None;
                    bitmapImage.CacheOption = BitmapCacheOption.OnLoad;
                    bitmapImage.UriSource = new Uri(CompletePath);
                    bitmapImage.EndInit();
                    bitmapImage.Freeze();
                }

                return bitmapImage;
            }
        }

        public string CompletePath
        {
            get
            {
                if (PicturePath == null)
                    return null;

                string path = Path.Combine(DataRoot.WorkingPath, PicturePath);
                if (!File.Exists(path))
                    return null;

                return path;
            }
        }

        public byte[] Image
        {
            set
            {
                string expansionFolder = Expansion.Name.Replace(":", "_").Replace("\\", "_").Replace("/", "_");
                string cardPicturePath = Path.Combine(DataRoot.WorkingPath, expansionFolder);
                if (!Directory.Exists(cardPicturePath))
                    Directory.CreateDirectory(cardPicturePath);
                string correctName = Name.Replace(":", "_").Replace("\\", "_").Replace("/", "_").Replace("?", "").Replace("!", "").Replace("\"", "") + Number + ".jpg";

                File.WriteAllBytes(Path.Combine(cardPicturePath, correctName), value);

                PicturePath = Path.Combine(expansionFolder, correctName);
                bitmapImage = null;


                OnPropertyChanged("Bitmap");
                BindPath();
            }
        }

        public void ExtractForceAndDefense()
        {
            if (Power == null)
            {
                Force = 0;
                Defense = 0;
                return;
            }

            int result;
            string[] splits = Power.Split('/');

            if (splits.Length < 2)
            {
                Force = 0;
                Defense = 0;
                return;
            }

            Force = !int.TryParse(splits[0], out result) ? 0 : result;

            Defense = !int.TryParse(splits[1], out result) ? 0 : result;
        }

        public void ForceRefresh()
        {
            OnPropertyChanged("Author");
            OnPropertyChanged("Rarity");
            OnPropertyChanged("Color");

            ExtractForceAndDefense();
        }

        public void BindPath()
        {
            BindPath(CompletePath);
        }

        public void BindPath(string path)
        {
            try
            {
                if (path == null)
                {
                    PictureFrame = null;
                    return;
                }
                using (FileStream fileStream = new FileStream(path, FileMode.Open))
                {
                    PictureFrame = (BitmapFrame)(BitmapFrame.Create(fileStream, BitmapCreateOptions.None, BitmapCacheOption.OnLoad).GetAsFrozen());
                }
            }
            catch
            {
                PictureFrame = null;
            }

            OnPropertyChanged("PictureFrame");
        }
    }
}
