﻿using System;
using System.Linq;
using System.Runtime.Serialization;

namespace UrzaIncubator.JSON
{
    [DataContract]
    public class JSONCard
    {
        [DataMember]
        public int id { get; set; }

        [DataMember]
        public int number { get; set; }

        [DataMember]
        public string nameEn { get; set; }

        [DataMember]
        public string nameFr { get; set; }

        [DataMember]
        public string author { get; set; }

        [DataMember]
        public int colorId { get; set; }

        [DataMember]
        public int rarityId { get; set; }

        [DataMember]
        public int? force { get; set; }

        [DataMember]
        public int? defense { get; set; }

        [DataMember]
        public string power { get; set; }

        [DataMember]
        public string picturePath { get; set; }

        [DataMember]
        public double? price { get; set; }

        [DataMember]
        public string textEn { get; set; }

        [DataMember]
        public string flavorEn { get; set; }

        [DataMember]
        public string textFr { get; set; }

        [DataMember]
        public string flavorFr { get; set; }

        [DataMember]
        public string typeEn { get; set; }

        [DataMember]
        public string typeFr { get; set; }

        [DataMember]
        public int multiverseIdEn { get; set; }

        [DataMember]
        public int multiverseIdFr { get; set; }

        [DataMember]
        public string manaCost { get; set; }

        [DataMember]
        public int? convertedManaCost { get; set; }

        [DataMember]
        public string tag { get; set; }

        [DataMember]
        public string cardMarketLink { get; set; }

        [DataMember]
        public JSONRuling[] rulings { get; set; }

        [DataMember]
        public JSONSize size { get; set; }

        public JSONCard(Card card)
        {
            id = card.ID;
            nameEn = card.Name.Replace("'", "’");
            nameFr = string.IsNullOrEmpty(card.FrenchName) ? "" : card.FrenchName.Replace("'", "’");
            author = card.Author.Name;
            colorId = card.Color.ID;
            defense = card.Defense;
            force = card.Force;
            number = card.Number;
            picturePath = card.PicturePath;
            power = card.Power;
            price = card.Price;
            rarityId = card.Rarity.ID;
            textEn = card.Text;
            flavorEn = card.Flavor;
            textFr = card.FrenchText;
            flavorFr = card.FrenchFlavor;
            typeEn = card.Type != null ? card.Type.Replace(": -", " -") : card.Type;
            typeFr = card.FrenchType != null ? card.FrenchType.Replace(": -", " -") : card.FrenchType;
            multiverseIdEn = card.MultiverseID ?? 0;
            multiverseIdFr = card.FrenchMultiverseID ?? 0;
            manaCost = card.ManaCost;
            convertedManaCost = card.ConvertedManaCost;
            tag = card.Tag;
            cardMarketLink = card.CardMarketLink;
            rulings = card.Rulings.OrderByDescending(r => r.date).Select(r => new JSONRuling(r)).ToArray();

            var substr = typeEn.Substring(0, Math.Min(7, typeEn.Length));
            size = new JSONSize
            {
                isPlane = (substr == "Plane -" || substr == "Plane —" || typeEn == "Phenomenon"),
                isScheme = (typeEn == "Scheme")
            };
        }

        public static implicit operator JSONCard(Card card)
        {
            return new JSONCard(card);
        }
    }
}
