﻿using System;
using System.Collections.Generic;
using System.Data.SqlServerCe;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.InteropServices;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Microsoft.Win32;
using UrzaIncubator.Rendering;
using mshtml;
using UrzaIncubator.JSON;
using System.Runtime.Serialization.Json;
using System.Diagnostics;
using System.Threading;
using System.Globalization;

namespace UrzaIncubator
{
    public static class Tools
    {
        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private static extern Int32 SystemParametersInfo(UInt32 uiAction, UInt32 uiParam, String pvParam, UInt32 fWinIni);
        private const UInt32 SPI_SETDESKWALLPAPER = 20;
        private const UInt32 SPIF_UPDATEINIFILE = 0x1;

        static readonly Random random = new Random((int)DateTime.Now.Ticks);

        static readonly Dictionary<string, string> expansionsSmallCodes = new Dictionary<string, string>();

        public static void TestConnection()
        {
            try
            {
                using (var sqlCeEngine = new SqlCeEngine((Entities.ConnectionString)))
                {
                    sqlCeEngine.Upgrade();
                }
            }
            catch
            {
            }


            using (var sqlConnection = new SqlCeConnection(Entities.ConnectionString))
            {
                sqlConnection.Open();

                // Patch
                try
                {
                    var command = new SqlCeCommand("ALTER TABLE EXPANSION ADD PriceUpdateIndex Integer", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD Text NVARCHAR(2048)", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD CardMarketLink NVARCHAR(2048)", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD Tag NVARCHAR(256)", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD MultiverseID INTEGER", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }


                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD FrenchMultiverseID INTEGER", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }


                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD ManaCost NVARCHAR(2048)", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD ConvertedManaCost INTEGER", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD FrenchType NVARCHAR(2048)", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD FrenchText NVARCHAR(2048)", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("ALTER TABLE CARD ADD Price FLOAT", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                try
                {
                    var command = new SqlCeCommand("CREATE TABLE RULING (id INTEGER IDENTITY, text nvarchar(2048), cardID integer, date DATETIME, PRIMARY KEY(id)) ", sqlConnection);
                    command.ExecuteNonQuery();
                }
                catch
                {
                }

                sqlConnection.Close();
            }
        }

        public static void GenerateDesktopBackground(Entities entities, string blockFilter, string expansionFilter)
        {
            try
            {
                IQueryable<Card> query;
                if (string.IsNullOrEmpty(expansionFilter) && string.IsNullOrEmpty(blockFilter))
                {
                    query = from c in entities.Cards orderby c.Name select c;
                }
                else if (string.IsNullOrEmpty(blockFilter))
                {
                    query = from c in entities.Cards where c.Expansion.Name == expansionFilter orderby c.Name select c;
                }
                else if (string.IsNullOrEmpty(expansionFilter))
                {
                    query = from c in entities.Cards where c.Expansion.Block.Name == blockFilter orderby c.Name select c;
                }
                else
                {
                    query = from c in entities.Cards where c.Expansion.Block.Name == blockFilter && c.Expansion.Name == expansionFilter orderby c.Name select c;
                }

                int total = query.Count();
                int cardNumber = random.Next(total);

                Card card = query.Skip(cardNumber).First();

                if (card.Bitmap.PixelWidth > card.Bitmap.PixelHeight)
                {
                    return;
                }
                double scale = 680.0 / card.Bitmap.PixelHeight;

                const int margin = 200;
                int width = (int)(card.Bitmap.PixelWidth * scale);
                int height = (int)(card.Bitmap.PixelHeight * scale);

                RenderTargetBitmap renderTarget = new RenderTargetBitmap(width + margin, height, 96.0, 96.0, PixelFormats.Default);

                DrawingVisual drawingVisual = new DrawingVisual();
                DrawingContext drawingContext = drawingVisual.RenderOpen();
                drawingContext.DrawRectangle(SystemColors.DesktopBrush, null, new Rect(0, 0, width + margin, height));
                drawingContext.DrawImage(card.Bitmap, new Rect(0, 0, width, height));


                Brush titleColor = Brushes.White;
                Brush textColor = Brushes.LightGray;
                DrawingZone drawingZone = new DrawingZone(width + 8, drawingContext, margin - 8);

                int row = 0;
                drawingZone.DrawText("Expansion:", 0, row++, 16, titleColor);
                drawingZone.DrawText(card.Expansion.Name, 10, row++, 15, textColor);
                drawingZone.DrawText("Block:", 0, row++, 16, titleColor);
                drawingZone.DrawText(card.Expansion.Block.Name, 10, row++, 15, textColor);
                drawingZone.DrawText("Rarity:", 0, row++, 16, titleColor);
                drawingZone.DrawText(card.Rarity.Name, 10, row++, 15, textColor);
                drawingZone.DrawText("Author:", 0, row++, 16, titleColor);
                drawingZone.DrawText(card.Author.Name, 10, row++, 15, textColor);
                drawingZone.DrawText("Number:", 0, row++, 16, titleColor);
                drawingZone.DrawText(string.Format("{0} / {1}", card.Number, card.Expansion.Cards.Count()), 10, row++, 15, textColor);
                drawingZone.DrawText("Global ID:", 0, row++, 16, titleColor);
                drawingZone.DrawText(string.Format("{0} / {1}", cardNumber, total), 10, row++, 15, textColor);
                drawingZone.DrawText("Price:", 0, row++, 16, titleColor);
                drawingZone.DrawText(string.Format("{0:0.00} €", card.Price), 10, row++, 15, textColor);
                drawingZone.DrawText("Status:", 0, row++, 16, titleColor);
                drawingZone.DrawText(card.Check ? "In collection" : "Missing", 10, row++, 15, card.Check ? Brushes.Green : Brushes.Red);
                row++;
                drawingZone.DrawText("French Translation:", 0, row++, 16, titleColor);
                drawingZone.DrawText(card.FrenchName, 0, row++, 15, titleColor);
                drawingZone.DrawText(card.FrenchFlavor, 0, row, 12, textColor);
                drawingContext.Close();

                renderTarget.Render(drawingVisual);

                string picturePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "urza.bmp");
                FileStream stream = new FileStream(picturePath, FileMode.Create);
                BmpBitmapEncoder encoder = new BmpBitmapEncoder();
                encoder.Frames.Add(BitmapFrame.Create(renderTarget));
                encoder.Save(stream);

                stream.Dispose();

                SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, picturePath, SPIF_UPDATEINIFILE);

                RegistryKey rkWallPaper = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop", true);
                rkWallPaper.SetValue("WallpaperStyle", 0);
                rkWallPaper.SetValue("TileWallpaper", 0);
                rkWallPaper.Close();
            }
            catch
            {
            }
        }

        public static bool GetCardImage(Card card)
        {
            string cardName = card.Name.ToLower();

            cardName = GetCardCorrectName(cardName, true);
            cardName = cardName.Replace(" ", "_");
            cardName = cardName.Replace("'", "");
            cardName = cardName.Replace("’", "");
            cardName = cardName.Replace("-", "_");
            cardName = cardName.Replace(",", "");
            cardName = cardName.Replace(":", "");

            string expansionName = card.Expansion.Name;
            expansionName = expansionName.Replace(" ", "_");
            expansionName = expansionName.Replace("'", "");
            expansionName = expansionName.Replace(":", "");

            WebRequest request = WebRequest.Create(string.Format("http://www.magiccardmarket.eu/img/cards/{0}/{1}.jpg", expansionName, cardName));

            try
            {
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                if (response.ContentType == "text/html")
                    return false;
                Stream stream = response.GetResponseStream();
                MemoryStream memStream = new MemoryStream();

                byte[] buffer = new byte[4096];
                int count;
                do
                {
                    count = stream.Read(buffer, 0, buffer.Length);
                    if (count > 0)
                        memStream.Write(buffer, 0, count);
                }
                while (count > 0);

                card.Image = memStream.GetBuffer();

                memStream.Dispose();
                stream.Dispose();
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static string GetCardCorrectName(string cardName, bool keepIndex)
        {
            cardName = cardName.Replace("û", "u");
            cardName = cardName.Replace("à", "a");
            cardName = cardName.Replace("á", "a");
            cardName = cardName.Replace("ö", "o");
            cardName = cardName.Replace("æ", "ae");
            cardName = cardName.Replace("’", "'");

            // Exceptions
            var exceptions = new[] { "jamuraan lion (version 1)", "jamuraan lion (version 2)" };

            if (exceptions.Contains(cardName))
            {
                keepIndex = false;
            }

            // Index
            for (int index = 1; index < 10; index++)
            {
                if (keepIndex)
                {
                    cardName = cardName.Replace(string.Format(" (v. {0})", index), index.ToString());
                    cardName = cardName.Replace(string.Format(" (ver. {0})", index), index.ToString());
                    cardName = cardName.Replace(string.Format(" (version {0})", index), index.ToString());
                    cardName = cardName.Replace(string.Format(" (v.{0})", index), index.ToString());
                }
                else
                {
                    cardName = cardName.Replace(string.Format(" (v. {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (ver. {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (version {0})", index), "");
                    cardName = cardName.Replace(string.Format(" (v.{0})", index), "");
                }
            }
            return cardName;
        }

        public static bool GetCardImage2(Card card, string language, string forcedTag = null, int forcedIndex = 0)
        {
            string expansionName;
            WebRequest rootRequest = WebRequest.Create("http://magiccards.info/sitemap.html");

            var sourceExpansionName = card.Expansion.Name;

            if (!expansionsSmallCodes.TryGetValue(sourceExpansionName, out expansionName))
            {
                if (sourceExpansionName == "Time Spiral - Timeshifted")
                {
                    expansionName = "tsts";
                }
                else if (sourceExpansionName == "Mirrodin")
                {
                    expansionName = "mi";
                }
                else if (sourceExpansionName == "Duel Decks: Phyrexia vs. the Coalition")
                {
                    expansionName = "pvc";
                }
                else
                {
                    string dataMap;
                    using (StreamReader reader = new StreamReader(rootRequest.GetResponse().GetResponseStream()))
                    {
                        dataMap = reader.ReadToEnd();
                    }

                    Regex regex =
                        new Regex(string.Format(@"({0}</a> <small style=""color: \#aaa;"">)(?<small>.+?)(</small>)",
                                                sourceExpansionName));

                    Match match = regex.Match(dataMap);

                    expansionName = match.Groups[3].Value;
                }
                expansionsSmallCodes.Add(sourceExpansionName, expansionName);
            }

            var tag = forcedTag ?? card.Tag;

            if (card.Name.Contains("//"))
            {
                tag = "a";
            }

            WebRequest request = WebRequest.Create(string.Format("http://magiccards.info/scans/{3}/{0}/{1}{2}.jpg", expansionName, forcedIndex != 0 ? forcedIndex : card.Number, tag, language));

            try
            {
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                if (response.ContentType == "text/html")
                    return false;
                Stream stream = response.GetResponseStream();
                MemoryStream memStream = new MemoryStream();

                byte[] buffer = new byte[4096];
                int count;
                do
                {
                    count = stream.Read(buffer, 0, buffer.Length);
                    if (count > 0)
                        memStream.Write(buffer, 0, count);
                }
                while (count > 0);

                card.Image = memStream.GetBuffer();

                memStream.Dispose();
                stream.Dispose();
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static IHTMLElement GetElementByInnerText(this IHTMLElement root, string innerText)
        {
            return ((IHTMLElementCollection)root.all).Cast<IHTMLElement>().FirstOrDefault(el => string.Compare(el.innerText, innerText, true) == 0);
        }

        public static IHTMLElement GetElementByClass(this IHTMLElement root, string id)
        {
            return ((IHTMLElementCollection)root.all).Cast<IHTMLElement>().FirstOrDefault(el => el.className != null && !string.IsNullOrEmpty(el.className) && el.className.ToLower() == id.ToLower());
        }

        public static string GetElementTextByClass(this IHTMLElement root, string id)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                var className = el.className;
                if (!string.IsNullOrEmpty(className) && className.ToLower() == id.ToLower())
                {
                    return el.innerText;
                }
            }

            return "";
        }

        public static IHTMLElement GetElementByTag(this IHTMLElement root, string tag, int order = 0)
        {
            int currentOrder = 0;
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (el.tagName.ToLower() == tag.ToLower())
                {
                    if (currentOrder == order)
                        return el;
                    currentOrder++;
                }
            }

            return null;
        }

        public static IHTMLElement GetElementByID(this IHTMLElement root, string id)
        {
            return (from IHTMLElement el in (IHTMLElementCollection)root.all let attribute = el.getAttribute("id") where attribute != null && attribute.ToString().ToLower().Contains(id.ToLower()) select el).FirstOrDefault();
        }

        public static string GetElementTextByClassIncomplete(this IHTMLElement root, string id)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (!string.IsNullOrEmpty(el.className) && el.className.ToLower().StartsWith(id.ToLower()))
                {
                    return el.innerText;
                }
            }

            return "";
        }

        public static string GetAnchorText(this IHTMLElement root)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (el is IHTMLAnchorElement && el.innerText != null)
                {
                    return el.innerText;
                }
            }

            return "";
        }

        public static string GetAnchorHref(this IHTMLElement root)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                IHTMLAnchorElement anchor = el as IHTMLAnchorElement;
                if (anchor != null && el.innerText != null)
                {
                    return anchor.href;
                }
            }

            return "";
        }

        public static IHTMLElement GetElementByClass(this IHTMLDocument2 document, string className)
        {
            return (from IHTMLElement el in (IHTMLElementCollection)document.body.all where el.className != null && el.className.ToLower() == className.ToLower() select el).FirstOrDefault();
        }

        public static IHTMLElement[] GetElementsByClass(this IHTMLDocument2 document, string className)
        {
            return (from IHTMLElement el in (IHTMLElementCollection)document.body.all where el.className != null && el.className.ToLower() == className.ToLower() select el).ToArray();
        }

        public static IHTMLElement[] GetElementsByPartOfClass(this IHTMLDocument2 document, string className)
        {
            return (from IHTMLElement el in (IHTMLElementCollection)document.body.all where el.className != null && el.className.ToLower().Contains(className.ToLower()) select el).ToArray();
        }

        public static IHTMLElement GetElementByID(this IHTMLDocument2 document, string id)
        {
            return (from IHTMLElement el in (IHTMLElementCollection)document.body.all let attribute = el.getAttribute("id") where attribute != null && attribute.ToString().ToLower() == id.ToLower() select el).FirstOrDefault();
        }

        public static IHTMLElement GetElementByPartOfID(this IHTMLDocument2 document, string id)
        {
            return (from IHTMLElement el in (IHTMLElementCollection)document.body.all let attribute = el.getAttribute("id") where attribute != null && attribute.ToString().ToLower().Contains(id.ToLower()) select el).FirstOrDefault();
        }

        internal static IHTMLElement GetAElementByInnerText(this IHTMLDocument2 document, string innerText, bool startOnly = false)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)document.body.all)
            {
                if (el.tagName == "A" && !string.IsNullOrEmpty(el.innerText))
                {
                    if (startOnly)
                    {
                        if (el.innerText.ToLower().StartsWith(innerText.ToLower()))
                            return el;
                    }
                    else if (string.Compare(el.innerText, innerText, true) == 0)
                        return el;
                }
            }

            return null;
        }

        internal static IHTMLElement GetImgElementByTitle(this IHTMLElement root, string title, bool startOnly = false)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)root.all)
            {
                if (el.tagName == "IMG")
                {
                    var attribute = el.getAttribute("title");

                    if (attribute != null)
                    {
                        if (startOnly)
                        {
                            if (attribute.ToString().ToLower().StartsWith(title.ToLower()))
                                return el;
                        }
                        else if (string.Compare(attribute.ToString(), title, true) == 0)
                            return el;
                    }
                }
            }

            return null;
        }

        internal static IHTMLElement GetImgElementByTitle(this IHTMLDocument2 document, string title, bool startOnly = false)
        {
            foreach (IHTMLElement el in (IHTMLElementCollection)document.body.all)
            {
                if (el.tagName == "IMG")
                {
                    var attribute = el.getAttribute("title");

                    if (attribute != null)
                    {
                        if (startOnly)
                        {
                            if (attribute.ToString().ToLower().StartsWith(title.ToLower()))
                                return el;
                        }
                        else if (string.Compare(attribute.ToString(), title, true) == 0)
                            return el;
                    }
                }
            }

            return null;
        }

        public static void GetCardMultiverseID(Card card)
        {
            if (card.MultiverseID != null)
                return;
            try
            {
                // Check
                card.Name = card.Name.Replace(" / ", " // ");

                // Go
                var cleanedName = card.Name.Replace("'s", " ").Replace("'", " ").Replace("’s", " ").Replace("’", " ").Replace(@"""", "").Replace("!", "").Replace("&", " ");
                string[] splits = cleanedName.Split();

                string requestName = "";
                string linkName = "";
                var expansionOffset = 0;
                var addReplace = "";

                foreach (string split in splits)
                {
                    if (split.StartsWith("(v.") || split.StartsWith("(ver."))
                    {
                        expansionOffset = int.Parse(splits[splits.Length - 1].Replace(")", "").Replace("(v.", "").Trim()) - 1;
                        break;
                    }
                    if (!string.IsNullOrEmpty(split))
                    {
                        linkName += string.IsNullOrEmpty(linkName) ? split : " " + split;
                        requestName += "+[" + split + "]";
                    }
                }

                if (card.Name.Contains("//"))
                {
                    var part = cleanedName.Split('/')[0].Trim();
                    linkName += " (" + part + ")";
                    addReplace = "&part=" + part;
                }

                string requestExpansion = string.Format(@"[""{0}""]", card.Expansion.Name);
                string url = string.Format(@"http://gatherer.wizards.com/Pages/Search/Default.aspx?action=advanced&special=true&name={0}&set={1}", requestName, requestExpansion);

                string data = GetData(url);
                IHTMLDocument2 document = (IHTMLDocument2)new HTMLDocument();
                document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                document.close();
                IHTMLElement root = document.GetElementByPartOfID("_otherSetsValue");

                if (root == null)
                {
                    string responseUrl;
                    int wait = 2000;

                    do
                    {
                        responseUrl = GetResponseUrl(url);

                        if (responseUrl == "http://gatherer.wizards.com/Pages/Error.aspx?aspxerrorpath=/Pages/Search/Default.aspx")
                        {
                            Debug.WriteLine("Waiting for " + card.Name);
                            Thread.Sleep(wait);
                            wait *= 2;
                        }
                    }
                    while (responseUrl == "http://gatherer.wizards.com/Pages/Error.aspx?aspxerrorpath=/Pages/Search/Default.aspx");

                    if (responseUrl.Contains("http://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid="))
                    {
                        card.MultiverseID = int.Parse(responseUrl.Replace("http://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid=", ""));
                        return;
                    }

                    IHTMLElement linkElement = (document.GetAElementByInnerText(linkName) ??
                                                document.GetAElementByInnerText(card.Name)) ?? document.GetAElementByInnerText(card.Name, true);

                    if (linkElement != null)
                    {
                        url = linkElement.getAttribute("href").ToString().Replace("..", "http://gatherer.wizards.com/Pages").Replace("about:", "");
                        data = GetData(url);
                        if (data != null)
                        {
                            document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                            document.close();

                            root = document.GetElementByPartOfID("_otherSetsValue");

                            if (root == null)
                            {
                                var multiverseIDText = url.Replace("http://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid=", "");

                                if (!string.IsNullOrEmpty(addReplace))
                                {
                                    multiverseIDText = multiverseIDText.Replace(addReplace, "");
                                }

                                card.MultiverseID = int.Parse(multiverseIDText);
                                return;
                            }
                        }
                    }
                }

                var val = new List<int>();
                foreach (dynamic element in root.children)
                {
                    if (element.children[0].title.Contains(card.Expansion.Name))
                    {
                        val.Add(int.Parse(element.href.Replace("about:Details.aspx?multiverseid=", "")));
                    }
                }

                if (val.Count > 0)
                {
                    val.Sort();

                    card.MultiverseID = val[expansionOffset];
                    return;
                }


                root = document.GetElementByPartOfID("_SetRow");
                if (root != null)
                {
                    foreach (dynamic element in root.children)
                    {
                        if (element.innerText.Contains(card.Expansion.Name))
                        {
                            card.MultiverseID = int.Parse(url.Replace("http://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid=", ""));
                            break;
                        }
                    }

                }
            }
            catch
            {
                var path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "err.txt");
                var text = string.Format("Unable to find multiverseID for :{0}({1})\r\n", card.Name, card.Expansion.Name);
                Debug.WriteLine(text);
                File.AppendAllText(path, text);
            }
        }

        public static void GetCardFrenchMultiverseID(Card card)
        {
            if (card.FrenchMultiverseID != null)
                return;
            try
            {
                string url = string.Format(@"http://gatherer.wizards.com/Pages/Card/Languages.aspx?multiverseid={0}", card.MultiverseID);

                string data = GetData(url);
                IHTMLDocument2 document = (IHTMLDocument2)new HTMLDocument();
                document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                document.close();

                IHTMLElement[] elements = document.GetElementsByPartOfClass("cardItem");

                foreach (var element in elements)
                {
                    if (element.children[1].innerText.Trim() == "French")
                    {
                        card.FrenchMultiverseID = int.Parse(element.children[0].children[0].href.Replace("about:Details.aspx?multiverseid=", ""));
                        return;
                    }
                }

                card.FrenchMultiverseID = 0; // Not found
            }
            catch
            {
                var path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "err.txt");
                var text = string.Format("Unable to find french multiverseID for :{0}({1})\r\n", card.Name, card.Expansion.Name);
                Debug.WriteLine(text);
                File.AppendAllText(path, text);
            }
        }

        public static void GetCardInfos(Card card, bool force = false)
        {
            if (card.MultiverseID == null)
            {
                GetCardMultiverseID(card);
            }

            if (card.FrenchMultiverseID == null)
            {
                GetCardFrenchMultiverseID(card);
            }

            if (!force && card.Type != null && card.Power != null && card.Flavor != null && card.FrenchFlavor != null && card.FrenchName != null && card.Text != null && card.ManaCost != null && card.ConvertedManaCost != null && card.FrenchType != null && card.FrenchText != null)
                return;
            try
            {
                string url = string.Format(@"http://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid={0}&printed=true", card.MultiverseID);

                string data = GetData(url);
                IHTMLDocument2 document = (IHTMLDocument2)new HTMLDocument();
                document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                document.close();
                IHTMLElement[] elements = document.GetElementsByClass("rightCol");

                if (elements.Length == 0)
                {
                    var path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "err.txt");
                    var text = string.Format("Unable to find card info for :{0}({1})\r\n", card.Name, card.Expansion.Name);
                    Debug.WriteLine(text);
                    File.AppendAllText(path, text);
                    return;
                }

                ProcessColumn(document, elements, card, 0, elements.Length != 1);
            }
            catch
            {
            }
        }

        private static void ProcessColumn(IHTMLDocument2 rootDocument, IHTMLElement[] list, Card card, int columnIndex, bool multiColumn)
        {
            IHTMLElement rootElement = list[columnIndex];
            IHTMLElement element;
            string data;

            if (multiColumn)
            {
                // Check name
                IHTMLElement cardNameElement = rootElement.GetElementByID("_nameRow");
                string cardName = ConvertHTMLElement2Text(cardNameElement.children[1]);

                if (card.Name != cardName)
                {
                    ProcessColumn(rootDocument, list, card, columnIndex + 1, true);
                    return;
                }
            }

            // Let's go
            if (card.Rulings.Count == 0)
            {
                IHTMLElement postContainerElement = rootDocument.GetElementByClass("postContainer");
                if (postContainerElement != null)
                {
                    IHTMLElement table = postContainerElement.children[0].children[0];

                    foreach (var child in table.children)
                    {
                        Ruling ruling = new Ruling();
                        ruling.Card = card;
                        card.Rulings.Add(ruling);

                        ruling.date = DateTime.Parse(child.children[0].innerText, CultureInfo.InvariantCulture);
                        ruling.text = ConvertHTMLElement2Text(child.children[1]);
                    }
                }
            }

            if (card.Type == null)
            {
                IHTMLElement typeElement = rootElement.GetElementByID("_typeRow");
                card.Type = typeElement.children[1].innerText.Trim().Replace("\r", "").Replace("\n", "");
                if (card.Color.Name == "Artifact" && card.Type.ToLower().Contains("land") && !card.Type.ToLower().Contains("artifact"))
                    card.Color = (from c in DataRoot.Entities.Colors where c.Name == "Land" select c).First();
            }

            if (card.Text == null)
            {
                IHTMLElement textElement = rootElement.GetElementByID("_textRow");
                if (textElement != null)
                    card.Text = ConvertHTML2Text(textElement.children[1].children);
                else
                    card.Text = "";
            }

            if (card.Power == null)
            {
                element = rootElement.GetElementByID("_ptRow");
                card.Power = element != null ? element.children[1].innerText.Trim().Replace("\r", "").Replace("\n", "") : "";
                card.ExtractForceAndDefense();
            }

            // Rarity
            IHTMLElement rarityElement = rootElement.GetElementByID("_rarityRow");
            if (rarityElement != null)
            {
                var rarity = ConvertHTML2Text(rarityElement.children[1].children).Replace("\r", "").Replace("\n", "");
                if (rarity == "Rare" && card.RarityID != 3)
                {
                    card.RarityID = 3;
                }
            }                

            // Flavor
            if (card.Flavor == null)
            {
                element = rootElement.GetElementByID("_FlavorText");

                if (element != null)
                {
                    card.Flavor = ExtractFlavor(element);
                }
                else
                {
                    card.Flavor = "";
                    card.FrenchFlavor = "";
                }
            }

            // Mana Cost
            if (card.ManaCost == null)
            {
                element = rootElement.GetElementByID("_ManaRow");

                if (element != null)
                {
                    card.ManaCost = ConvertHTMLElement2Text(element.children[1]);
                }
                else
                {
                    card.ManaCost = "";
                }
            }

            // Converted Mana Cost
            if (card.ConvertedManaCost == null)
            {
                element = rootElement.GetElementByID("_cmcRow");

                if (element != null)
                {
                    card.ConvertedManaCost = int.Parse(element.children[1].innerText.Trim());
                }
                else
                {
                    card.ConvertedManaCost = 0;
                }
            }

            // French flavor
            if (card.FrenchMultiverseID != 0)
            {
                if (card.FrenchFlavor == null || card.FrenchName == null || card.FrenchText == null || card.FrenchType == null)
                {
                    string url = string.Format(@"http://gatherer.wizards.com/Pages/Card/Details.aspx?printed=true&multiverseid={0}", card.FrenchMultiverseID);

                    data = GetData(url);
                    IHTMLDocument2 document = (IHTMLDocument2)new HTMLDocument();
                    document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                    document.close();
                    IHTMLElement[] elements = document.GetElementsByClass("rightCol");

                    if (elements.Length == 0)
                    {
                        var path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "err.txt");
                        var text = string.Format("Unable to find french card info for :{0}({1})\r\n", card.Name, card.Expansion.Name);
                        Debug.WriteLine(text);
                        File.AppendAllText(path, text);
                        return;
                    }

                    ProcessColumnFrench(elements[columnIndex], card);
                }
            }
        }

        private static void ProcessColumnFrench(IHTMLElement rootElement, Card card)
        {
            IHTMLElement element;

            // Name
            if (card.FrenchName == null)
            {
                IHTMLElement textElement = rootElement.GetElementByID("_nameRow");
                card.FrenchName = textElement.children[1].innerText.Trim();
            }

            // Type
            if (card.FrenchType == null)
            {
                IHTMLElement typeElement = rootElement.GetElementByID("_typeRow");
                card.FrenchType = typeElement.children[1].innerText.Trim().Replace("\r", "").Replace("\n", "");
            }

            // Text
            if (card.FrenchText == null)
            {
                IHTMLElement textElement = rootElement.GetElementByID("_textRow");

                if (textElement != null)
                {
                    card.FrenchText = ConvertHTML2Text(textElement.children[1].children);
                }
                else
                {
                    card.FrenchText = "";
                }
            }

            // Flavor
            if (card.FrenchFlavor == null)
            {
                element = rootElement.GetElementByID("_FlavorText");

                if (element != null)
                {
                    card.FrenchFlavor = ExtractFlavor(element);
                }
                else
                {
                    card.FrenchFlavor = "";
                }
            }
        }

        private static string ConvertHTML2Text(dynamic elements)
        {
            string result = "";
            foreach (IHTMLElement element in elements)
            {
                result += ConvertHTMLElement2Text(element) + "\r\n";
            }

            return result;
        }

        static string ConvertHTMLElement2Text(IHTMLElement root)
        {
            if (root.children.length == 0)
                return root.innerText;

            string result = root.innerHTML;
            foreach (IHTMLElement element in root.children)
            {
                if (element.tagName == "IMG")
                {
                    string replaceString = "";

                    for (int index = 0; index < 17; index++)
                    {
                        if (element.outerHTML.Contains("name=" + index.ToString()))
                            replaceString = "(" + index.ToString() + ")";
                    }

                    if (element.outerHTML.Contains("name=500"))
                        replaceString = "(500)";

                    if (element.outerHTML.Contains("name=100"))
                        replaceString = "(100)";

                    if (element.outerHTML.Contains("name=Infinity"))
                        replaceString = "(Infinity)";

                    if (element.outerHTML.Contains("name=X"))
                        replaceString = "(X)";

                    if (element.outerHTML.Contains("name=tap"))
                        replaceString = "(T)";

                    if (element.outerHTML.Contains("name=untap"))
                        replaceString = "(-T)";

                    if (element.outerHTML.Contains("name=W"))
                        replaceString = "(W)";

                    if (element.outerHTML.Contains("name=R"))
                        replaceString = "(R)";

                    if (element.outerHTML.Contains("name=G"))
                        replaceString = "(G)";

                    if (element.outerHTML.Contains("name=U"))
                        replaceString = "(U)";

                    if (element.outerHTML.Contains("name=B"))
                        replaceString = "(B)";

                    if (element.outerHTML.Contains("name=2W"))
                        replaceString = "(2W)";

                    if (element.outerHTML.Contains("name=2R"))
                        replaceString = "(2R)";

                    if (element.outerHTML.Contains("name=2G"))
                        replaceString = "(2G)";

                    if (element.outerHTML.Contains("name=2U"))
                        replaceString = "(2U)";

                    if (element.outerHTML.Contains("name=2B"))
                        replaceString = "(2B)";

                    if (element.outerHTML.Contains("name=WP"))
                        replaceString = "(WP)";

                    if (element.outerHTML.Contains("name=RP"))
                        replaceString = "(RP)";

                    if (element.outerHTML.Contains("name=GP"))
                        replaceString = "(GP)";

                    if (element.outerHTML.Contains("name=UP"))
                        replaceString = "(UP)";

                    if (element.outerHTML.Contains("name=BP"))
                        replaceString = "(BP)";

                    if (element.outerHTML.Contains("name=UB"))
                        replaceString = "(UB)";

                    if (element.outerHTML.Contains("name=UR"))
                        replaceString = "(UR)";

                    if (element.outerHTML.Contains("name=RW"))
                        replaceString = "(RW)";

                    if (element.outerHTML.Contains("name=RG"))
                        replaceString = "(RG)";

                    if (element.outerHTML.Contains("name=GU"))
                        replaceString = "(GU)";

                    if (element.outerHTML.Contains("name=GW"))
                        replaceString = "(GW)";

                    if (element.outerHTML.Contains("name=WU"))
                        replaceString = "(WU)";

                    if (element.outerHTML.Contains("name=WB"))
                        replaceString = "(WB)";

                    if (element.outerHTML.Contains("name=BR"))
                        replaceString = "(BR)";

                    if (element.outerHTML.Contains("name=BG"))
                        replaceString = "(BG)";

                    result = result.Replace(element.outerHTML, replaceString);
                }
                else
                {
                    result = result.Replace(element.outerHTML, element.innerText);
                }
            }

            return result;
        }

        private static string ExtractFlavor(IHTMLElement element)
        {
            string flavor = "";
            foreach (IHTMLElement child in element.children)
            {
                if (child.innerText != null)
                {
                    string value = child.innerText.Trim().Replace("\r", "").Replace("\n", "");

                    if (!string.IsNullOrEmpty(value))
                        flavor += value + "\n";
                }
            }
            return flavor;
        }

        public static string GetData(string url)
        {
            if (!url.StartsWith("https://", StringComparison.InvariantCultureIgnoreCase) && !url.StartsWith("http://", StringComparison.InvariantCultureIgnoreCase))
                return null;

            int retry = 0;
            do
            {
                try
                {
                    HttpWebRequest rootRequest = (HttpWebRequest)WebRequest.Create(url);
                    rootRequest.UserAgent = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)";

                    string data;
                    var response = rootRequest.GetResponse();
                    using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                    {
                        data = reader.ReadToEnd();
                    }
                    return data.Replace("<I>", "").Replace("<i>", "").Replace("</I>", "").Replace("</i>", ""); ;
                }
                catch
                {
                    retry++;
                    Thread.Sleep(2000);
                }
            }
            while (retry < 2);

            return null;
        }

        public static string GetResponseUrl(string url)
        {
            if (!url.StartsWith("http://", StringComparison.InvariantCultureIgnoreCase))
                return null;

            int retry = 0;
            do
            {
                try
                {
                    HttpWebRequest rootRequest = (HttpWebRequest)WebRequest.Create(url);
                    var response = rootRequest.GetResponse();
                    var result = response.ResponseUri.ToString();

                    response.Close();

                    return result;
                }
                catch
                {
                    retry++;
                }
            }
            while (retry < 5);

            return null;
        }

        public static void CleanAuthors(Entities urzaGathererEntities)
        {
            Author[] authors = urzaGathererEntities.Authors.ToArray();

            foreach (Author author in authors)
            {
                if (author.EntityState == System.Data.EntityState.Deleted)
                    continue;
                Author author1 = author;
                var query = from a in urzaGathererEntities.Authors
                            where string.Compare(a.Name, author1.Name, true) == 0 && a.ID != author1.ID
                            select a;

                List<Author> duplicates = query.ToList();
                foreach (Author duplicate in duplicates)
                {
                    List<Card> cards = duplicate.Cards.ToList();
                    foreach (Card c in cards)
                    {
                        c.Author = author;
                    }
                    urzaGathererEntities.DeleteObject(duplicate);
                }
            }
            urzaGathererEntities.SaveChanges();
        }

        public static void GenerateMinifiedJSON()
        {
            var desktop = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);

            using (
                var stream = new FileStream(Path.Combine(desktop, "all.minified.json"), FileMode.OpenOrCreate,
                                            FileAccess.Write))
            {
                var blocks = new List<JSON.Minified.JSONBlock>();

                foreach (var block in DataRoot.Entities.Blocks)
                {
                    if (block.Name != "Oracle")
                        blocks.Add(new JSON.Minified.JSONBlock(block));
                }

                var ser = new DataContractJsonSerializer(typeof(JSON.Minified.JSONBlock[]));
                ser.WriteObject(stream, blocks.ToArray());
            }
        }

        static void ExportImageStream(string name, string folder, byte[] data, int requiredWidth, int requiredHeight)
        {
            var desktop = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
            BitmapImage result = new BitmapImage();

            result.BeginInit();
            int? width = null;
            int? height = null;

            using (MemoryStream ms = new MemoryStream(data))
            {
                result.StreamSource = ms;
                result.CreateOptions = BitmapCreateOptions.None;
                result.CacheOption = BitmapCacheOption.Default;
                result.EndInit();

                if (result.PixelWidth > result.PixelHeight)
                    width = requiredWidth;
                else
                    height = requiredHeight;

            }

            result = new BitmapImage();

            result.BeginInit();

            if (width.HasValue)
                result.DecodePixelWidth = width.Value;
            else
                result.DecodePixelHeight = height.Value;

            using (MemoryStream ms = new MemoryStream(data))
            {
                result.StreamSource = ms;
                result.CreateOptions = BitmapCreateOptions.None;
                result.CacheOption = BitmapCacheOption.Default;
                result.EndInit();
                var desktopLogos = Path.Combine(desktop, folder + "\\");

                if (!Directory.Exists(desktopLogos))
                    Directory.CreateDirectory(desktopLogos);

                using (FileStream outputFile = File.OpenWrite(Path.Combine(desktopLogos + name.Replace(":", "_") + ".png")))
                {
                    PngBitmapEncoder encoder = new PngBitmapEncoder();
                    encoder.Frames.Add(BitmapFrame.Create(result));
                    encoder.Save(outputFile);
                }
            }
        }

        public static void GenerateJSON()
        {
            var desktop = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);

            using (var stream = new FileStream(Path.Combine(desktop, "all.json"), FileMode.OpenOrCreate, FileAccess.Write))
            {
                var blocks = new List<JSONBlock>();

                foreach (var block in DataRoot.Entities.Blocks)
                {
                    if (block.Name == "Oracle")
                        continue;

                    foreach (Expansion exp in block.Expansions)
                    {
                        if (exp.Logo != null)
                        {
                            ExportImageStream(exp.Name, "Logos", exp.Logo, 250, 90);
                        }

                        if (exp.Image != null)
                        {
                            ExportImageStream(exp.Name, "Icons", exp.Image, 32, 32);
                        }
                    }
                    blocks.Add(new JSONBlock(block));
                }

                DataContractJsonSerializer ser = new DataContractJsonSerializer(typeof(JSONBlock[]));
                ser.WriteObject(stream, blocks.ToArray());
            }
        }
    }
}
